@@ -40,19 +40,19 @@ for k, v in params_beamline.items():
 
 
 class CDIRunnerScanID10(CDIRunnerScan):
-    def __init__(self, params, scan):
-        super(CDIRunnerScanID10, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(CDIRunnerScanID10, self).__init__(params, scan, timings=timings)
 
-    def prepare_cdi(self, free_pixel_mask=None):
+    def prepare_cdi(self):
         """
         Prepare CDI object from input data.
 
-        :param free_pixel_mask: optionally supply a free pixel mask to keep it for successive runs
-        :return: nothing. Creates self.cdi object.
+        :return: nothing. Creates or updates self.cdi object.
         """
-        super(CDIRunnerScanID10, self).prepare_cdi(free_pixel_mask=free_pixel_mask)
-        # Scale initial object
-        self.cdi = ScaleObj(method='F') * self.cdi
+        super(CDIRunnerScanID10, self).prepare_cdi()
+        # Scale initial object (unnecessary if auto-correlation is used)
+        if self.params['support'] != 'auto':
+            self.cdi = ScaleObj(method='F', lazy=True) * self.cdi
 
 
 class CDIRunnerID10(CDIRunner):
