@@ -15,11 +15,10 @@ import timeit
 import numpy as np
 from scipy.optimize import minimize
 from scipy.ndimage import fourier_shift
+from scipy.ndimage.measurements import center_of_mass
 from scipy.fftpack import fftn, ifftn, fftshift
-try:
-    from skimage.registration import phase_cross_correlation as register_translation
-except ImportError:
-    from skimage.feature import register_translation
+
+from ..utils.registration import phase_cross_correlation
 from ..processing_unit import has_cuda, has_opencl
 from ..processing_unit import default_processing_unit as main_default_processing_unit
 import warnings
@@ -45,31 +44,84 @@ elif has_opencl and main_default_processing_unit.pu_language not in ['cuda', 'cp
     from ..processing_unit.kernel_source import get_kernel_source as getks
 
 
-def match_shape(arrays, method='min'):
+def array_cen(a: np.ndarray, thres=None, decomposed=False):
+    """
+    Move the centre of mass of the array to its centre.
+    :param a: the array to be centered
+    :param thres: if a fraction (between 0 and 1) is given, the center of mass is
+        calculated on an array whare all values are capped to abs(a).max*thres,
+        so that a few intense pixels do not have an exaggerated weight.
+    :param decomposed: if True, the centre of mass and thresholding is done by projecting
+        independently against each axis. This is better at avoiding that an object with
+        an asymmetric shape (e.g. a triangle) gets rolled out on the sides of the array.
+        Ignored if thres is None.
+    :return: the centred array
+    """
+    if thres is None:
+        c = center_of_mass(abs(a))
+    else:
+        if decomposed:
+            c = []
+            for i in range(a.ndim):
+                ax = list(range(a.ndim))
+                ax.pop(i)
+                aa = abs(a).sum(axis=tuple(ax))
+                m = aa.max() * thres
+                aa[aa > m] = m
+                c.append(center_of_mass(aa)[0])
+        else:
+            aa = abs(a)
+            m = aa.max() * thres
+            aa[aa > m] = m
+            c = center_of_mass(aa)
+    dr = np.array(a.shape).astype(np.float32) / 2 - np.array(c)
+    dr = np.round(dr).astype(np.int32)
+    if np.allclose(dr, 0):
+        return a
+    return np.roll(a, dr, range(a.ndim))
+
+
+def match_shape(arrays, method='min', shape=None, cen=True):
     """
     Match the shape of two or more arrays by cropping the borders, or zero-padding if necessary
     :param arrays: a list or tuple of arrays, all either 2D or 3D
     :param method: either 'min' (use the largest size along each dimension which is smaller than all array sizes)
+        or 'max' (largest size along each dimension among all the arrays)
         or 'median': use the median value for the size along each dimension. The 'median' option is better when
         matching more than 2 arrays, when one may be an outlier with incorrect dimensions.
+    :param shape: if given, all arrays shapes will be set to the given tuple, and 'method' is ignored.
+    :param cen: if True, the arrays will be centred (using a 0.1 threshold and decomposition) before matching
     :return: a list of the arrays cropped or zero-padded to the same shape. The data type of each individual array
         is preserved
     """
     d1 = arrays[0]
     v = []
     ndim = d1.ndim
-    if method == 'median':
-        nx = int(np.median(list(d.shape[-1] for d in arrays)))
-        ny = int(np.median(list(d.shape[-2] for d in arrays)))
-        if ndim == 3:
-            nz = int(np.median(list(d.shape[-3] for d in arrays)))
+    if shape is not None:
+        if len(shape) == 3:
+            nz, ny, nx = shape
+        elif len(shape) == 2:
+            ny, nx = shape
     else:
-        nx = min((d.shape[-1] for d in arrays))
-        ny = min((d.shape[-2] for d in arrays))
-        if ndim == 3:
-            nz = min((d.shape[-3] for d in arrays))
+        if method == 'median':
+            nx = int(np.median(list(d.shape[-1] for d in arrays)))
+            ny = int(np.median(list(d.shape[-2] for d in arrays)))
+            if ndim == 3:
+                nz = int(np.median(list(d.shape[-3] for d in arrays)))
+        elif method == 'max':
+            nx = max((d.shape[-1] for d in arrays))
+            ny = max((d.shape[-2] for d in arrays))
+            if ndim == 3:
+                nz = max((d.shape[-3] for d in arrays))
+        else:
+            nx = min((d.shape[-1] for d in arrays))
+            ny = min((d.shape[-2] for d in arrays))
+            if ndim == 3:
+                nz = min((d.shape[-3] for d in arrays))
 
     for d in arrays:
+        if cen:
+            d = array_cen(d, thres=0.1, decomposed=True)
         if ndim == 3:
             nz1, ny1, nx1 = d.shape
             tmp = np.zeros((nz, ny, nx), dtype=d.dtype)
@@ -108,7 +160,8 @@ def flipn(d: np.ndarray, flip_axes):
     Flip an array along any combination of axes
     :param d: the array to manipulate
     :param flip_axes: an iterable list of d.ndim values with True/False values indicating if the array must be flipped
-                      along each axis. If flip_axes=None, flip is dona along all axes
+                      along each axis. If flip_axes=None, flip is dona along all axes.
+                      An extra value can be added to optionally return the conjugate of the array.
     :return: a copy of the array after modification
     """
     if flip_axes is None:
@@ -116,6 +169,9 @@ def flipn(d: np.ndarray, flip_axes):
     for i in range(d.ndim):
         if flip_axes[i]:
             d = np.flip(d, i)
+    if len(flip_axes) > d.ndim:
+        if flip_axes[d.ndim]:
+            return d.conj()
     return d
 
 
@@ -153,7 +209,7 @@ def fit_phase2(pars, phi1: np.ndarray, phi2: np.ndarray, a1: np.ndarray, a2: np.
     dphi = phi2 - pars[0]
     for i in range(len(xyz)):
         dphi -= pars[i + 1] * xyz[i]
-    dphi = abs(phi1 - dphi)
+    dphi = np.abs(phi1 - dphi)
     dphi = np.minimum(dphi, 2 * np.pi - dphi)
     return (a1 * a2 * dphi ** 2).sum()
 
@@ -218,7 +274,7 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
     :param d1: the first array
     :param d2: the second array
     :param match_phase_ramp: if True (the default), the phase ramps will be matched as well. This is slower. The
-                             phase shift id always fitted.
+                             phase shift is always fitted, unless both arrays are real.
     :param match_orientation: either 'center' or 'all', to test only the center-of-symmetry inversion or a flip along
                               all axes
     :param match_scale: if True (the default), multiply d2 by a scale factor to match the amplitude of d1
@@ -242,7 +298,8 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
     """
     # Match the two arrays shape
     t0 = timeit.default_timer()
-    d1, d2 = match_shape((d1, d2))
+    if d1.shape != d2.shape:
+        d1, d2 = match_shape((d1, d2), cen=False)
     t1 = timeit.default_timer()
     if verbose >= 2:
         print("Match2: dt[match_shape]=%6.3fs" % (t1 - t0))
@@ -257,7 +314,7 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
             for flipx in [0, 1]:
                 for flipy in [0, 1]:
                     for flipz in [0, 1]:
-                        s, err, dphi = register_translation(a1, flipn(a2, [flipz, flipy, flipx]))
+                        s, err, dphi = phase_cross_correlation(a1, flipn(a2, [flipz, flipy, flipx]))
                         if errmin > err:
                             flip_min = [flipz, flipy, flipx]
                             errmin = err
@@ -268,7 +325,7 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
         else:
             for flipx in [0, 1]:
                 for flipy in [0, 1]:
-                    s, err, dphi = register_translation(a1, flipn(a2, [flipy, flipx]))
+                    s, err, dphi = phase_cross_correlation(a1, flipn(a2, [flipy, flipx]))
                     if errmin > err:
                         flip_min = [flipy, flipx]
                         errmin = err
@@ -279,7 +336,7 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
     else:
         if d1.ndim == 3:
             for flip in [0, 1]:
-                s, err, dphi = register_translation(a1, flipn(a2, [flip, flip, flip]))
+                s, err, dphi = phase_cross_correlation(a1, flipn(a2, [flip, flip, flip]))
                 if errmin > err:
                     flip_min = [flip, flip, flip]
                     errmin = err
@@ -289,7 +346,7 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
                     flip_min[0], flip_min[1], flip_min[2], errmin))
         else:
             for flip in [0, 1]:
-                s, err, dphi = register_translation(a1, flipn(a2, [flip, flip]))
+                s, err, dphi = phase_cross_correlation(a1, flipn(a2, [flip, flip]))
                 if errmin > err:
                     flip_min = [flip, flip]
                     errmin = err
@@ -305,13 +362,13 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
     conj = False
     phase_shift = None
     if match_phase_ramp:
-        pixel_shift, err, dphi = register_translation(a1, abs(d2), upsample_factor=upsample_factor)
+        pixel_shift, err, dphi = phase_cross_correlation(a1, abs(d2), upsample_factor=upsample_factor)
         # Roll can be used for pixel registration
         # d2 = np.roll(d2, [int(round(v)) for v in s], range(d2.ndim)) * np.exp(1j * dphi)
         d2 = ifftn(fourier_shift(fftn(d2), pixel_shift))
         t3 = timeit.default_timer()
         if verbose >= 2:
-            print("Match2: dt[register_translation]=%6.3fs" % (t3 - t2))
+            print("Match2: dt[registration]=%6.3fs" % (t3 - t2))
 
         # Match phase shift and ramp, testing both conjugates
         if has_cuda and use_gpu:
@@ -412,12 +469,13 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
 
             p0 = np.zeros(1 + d1.ndim)
             p0[0] = dphi
-            p0 = minimize(fit_phase2, p0, args=(phi1, phi2, a1, a2, xyz), method='powell')
+            options = {'xtol': 1e-3, 'ftol': 1e-3}
+            p0 = minimize(fit_phase2, p0, args=(phi1, phi2, a1, a2, xyz), method='powell', options=options)
             if verbose >= 3:
                 print(p0)
             p1 = np.zeros(1 + d1.ndim)
             p1[0] = -dphi
-            p1 = minimize(fit_phase2, p1, args=(phi1, -phi2, a1, a2, xyz), method='powell')
+            p1 = minimize(fit_phase2, p1, args=(phi1, -phi2, a1, a2, xyz), method='powell', options=options)
             if verbose >= 3:
                 print(p1)
         if p0['fun'] < p1['fun']:
@@ -431,18 +489,25 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
         if verbose >= 2:
             print("Match2: dt[match_phase_ramp]=%6.3fs" % (t4 - t3), "ramp: ", phase_shift)
     else:
-        pixel_shift, err, dphi = register_translation(d1, d2, upsample_factor=upsample_factor)
+        if np.sum(flip_min) % 2 and np.iscomplexobj(d1):
+            d2 = d2.conj()
+        pixel_shift, err, dphi = phase_cross_correlation(d1, d2, upsample_factor=upsample_factor)
+        if verbose >= 2:
+            print("Match2: ", pixel_shift, err, dphi)
         phase_shift = np.zeros(d1.ndim + 1)
         phase_shift[0] = -dphi
         # Roll can be used for pixel registration
         # d2 = np.roll(d2, [int(round(v)) for v in s], range(d2.ndim)) * np.exp(1j * dphi)
-        d2 = ifftn(fourier_shift(fftn(d2), pixel_shift)) * np.exp(1j * -dphi)
+        if np.isrealobj(d1) and np.isrealobj(d2):
+            d2 = abs(ifftn(fourier_shift(fftn(d2), pixel_shift))).astype(np.float32)
+        else:
+            d2 = ifftn(fourier_shift(fftn(d2), pixel_shift)) * np.exp(1j * dphi)
         t4 = t3 = timeit.default_timer()
         if verbose >= 2:
-            print("Match2: dt[register_translation]=%6.3fs" % (t3 - t2))
+            print("Match2: dt[registration]=%6.3fs" % (t3 - t2))
 
     if verbose:
-        print(register_translation(a1, abs(d2)))
+        print(phase_cross_correlation(a1, abs(d2)))
 
     if match_scale:
         # Match amplitudes to minimise (abs(a1-S*a2)**2).sum()
