@@ -6,19 +6,23 @@
 #         Vincent Favre-Nicolin, favre@esrf.fr
 
 import os
+import platform
 import warnings
+import timeit
+from abc import abstractmethod
+from psutil import cpu_count
 from .cuda_device import has_cuda
 from .opencl_device import has_opencl
 import numpy as np
-from scipy.fftpack import fftn, ifftn, fftshift
+# scipy.fft must be explicitly imported
+import scipy.fft
+from scipy.fft import fftn, ifftn, rfftn, irfftn
 
-try:
-    from mpi4py import MPI
+from ..mpi import MPI
 
+if MPI is not None:
     mpi = MPI.COMM_WORLD
-    if mpi.Get_size() == 1:
-        mpi = None
-except ImportError:
+else:
     mpi = None
 
 
@@ -26,6 +30,10 @@ class ProcessingUnitException(Exception):
     pass
 
 
+class ProcessingUnitWarning(UserWarning):
+    pass
+
+
 # Exclude languages based on PYNX_PU environment variable
 if 'PYNX_PU' in os.environ:
     pynx_py_env = os.environ['PYNX_PU'].lower().split('.')
@@ -52,7 +60,6 @@ class ProcessingUnit(object):
     def __init__(self):
         # Generic parameters
         self.pu_language = None  # 'cpu', 'opencl' or 'cuda' - None if un-initialised
-        self.max_prime_fft = None
         # OpenCL device
         self.cl_device = None
         # CUDA device
@@ -62,14 +69,20 @@ class ProcessingUnit(object):
         self.benchmark_fft_size = (16, 400, 400)
         # if True, will perform a batch FFT (e.g. 16 2D FFT of size 256x256 with the default parameters)
         self.benchmark_fft_batch = True
+        try:
+            # Get the real number of processor cores available
+            # os.sched_getaffinity is only available on some *nix platforms
+            self.nproc = len(os.sched_getaffinity(0)) * cpu_count(logical=False) // cpu_count(logical=True)
+        except AttributeError:
+            self.nproc = os.cpu_count()
 
     def enable_profiling(self, profiling=True):
         """
-        Enable profiling (for OpenCL only)
+        Enable profiling (availability may depend on the backend)
         :param profiling: True to enable (the default)
         :return:
         """
-        print('Profiling can only be enabled when using OpenCL')
+        pass
 
     def set_benchmark_fft_parameters(self, fft_size, batch):
         """
@@ -93,7 +106,6 @@ class ProcessingUnit(object):
         """
         if has_opencl:
             from . import opencl_device
-            from .opencl_utils import gpyfft_max_prime
             if self.benchmark_fft_batch:
                 fft_axis = (-2, -1)
             else:
@@ -103,7 +115,7 @@ class ProcessingUnit(object):
                     opencl_device.cl_device_fft_speed(d, self.benchmark_fft_size, fft_axis, True)
                 self.cl_device = d
                 self.pu_language = 'opencl'
-                self.max_prime_fft = gpyfft_max_prime(d)
+
                 if verbose:
                     print("Using OpenCL GPU: %s" % (self.cl_device.name))
                 return True
@@ -112,7 +124,6 @@ class ProcessingUnit(object):
             if type(d) is cuda_device.cu_drv.Device:
                 self.cu_device = d
                 self.pu_language = 'cuda'
-                self.max_prime_fft = 7
                 if verbose:
                     print("Using CUDA GPU: %s" % (self.cu_device.name()))
                 return True
@@ -199,7 +210,7 @@ class ProcessingUnit(object):
                       DeprecationWarning)
         return self.select_gpu(gpu_name=gpu_name, gpu_rank=gpu_rank, ranking=ranking, language=language)
 
-    def select_gpu(self, gpu_name=None, gpu_rank=0, ranking="fft_speed", language=None, verbose=False):
+    def select_gpu(self, gpu_name=None, gpu_rank=0, ranking="bandwidth", language=None, verbose=False):
 
         """
         Automatically select the fastest GPU, either from CUDA or OpenCL.
@@ -220,10 +231,13 @@ class ProcessingUnit(object):
         :param gpu_rank: the rank of the GPU to use. If ranking is 'fft_speed', this will select from the fastest
                          GPU. If ranking is 'order', it will just find the GPU matching the name, in the order
                          they are found.
-        :param ranking: default is 'fft_speed' (or 'fft'), all GPU are tested for FFT speed, and chosen from that
+        :param ranking: can be 'fft_speed' (or 'fft'), all GPU are tested for FFT speed, and chosen from that
                         ranking. If ranking is 'order', they will be listed as they are found by CUDA/OpenCL.
+                        If 'bandwidth' (default), the on-device memory transfer speed is used as benchmark.
                         This is useful to select a GPU just based on its name, without the overhead of the FFT test.
-                        Ignored if using MPI.
+                        Ignored if using MPI (the ranking will use the GPU's PCI id (cuda) or pointer (opencl),
+                        distributed by MPI order to the process on the same node. Note that it will be
+                        cleaner if each MPI process only sees a single GPU).
         :param language: either 'opencl', 'cuda', 'cpu', or None. If None, the preferred language is cuda>opencl>cpu
         :param verbose: True or False (default) for verbosity
         :return: True if a GPU (or CPU) could be selected. An exception is returned if no GPU was found,
@@ -235,6 +249,7 @@ class ProcessingUnit(object):
         benchmark_results = []
 
         if 'PYNX_PU' in os.environ:
+            pynx_py_env = os.environ['PYNX_PU'].lower().split('.')
             if pynx_py_env[0] not in ['opencl', 'cuda', 'cpu']:
                 if len(pynx_py_env[0]) > 0:
                     gpu_name = pynx_py_env[0]
@@ -267,6 +282,7 @@ class ProcessingUnit(object):
             # This will deactivate FFT speed, just reporting found GPU devices
             fft_shape = None
         else:
+            ranking = 'fft'
             fft_shape = self.benchmark_fft_size
 
         test_cuda = False
@@ -280,7 +296,8 @@ class ProcessingUnit(object):
             if has_cuda:
                 benchmark_results += cuda_device.available_gpu_speed(fft_shape=fft_shape,
                                                                      batch=self.benchmark_fft_batch, min_gpu_mem=None,
-                                                                     verbose=verbose, gpu_name=gpu_name)
+                                                                     verbose=verbose, gpu_name=gpu_name,
+                                                                     ranking=ranking)
             if language is not None:
                 if 'cuda' in language.lower():
                     if len(benchmark_results) == 0:
@@ -289,11 +306,38 @@ class ProcessingUnit(object):
                             s += ' [gpu_name=%s]' % str(gpu_name)
                         raise Exception(s)
 
-        if len(benchmark_results) and mpi is not None:
-            # if we have more than 1 CUDA GPU available, use MPI rank to select it
-            nb = len(benchmark_results)
-            r = mpi.Get_rank()
-            return self.set_device(benchmark_results[r % nb][0], test_fft=False, verbose=verbose)
+        if len(benchmark_results):
+            # We have found at least 1 CUDA device, so use it
+            if mpi is not None:
+                nb = len(benchmark_results)
+                r = mpi.Get_rank()
+                if nb > 1:
+                    # TODO: evaluate how reliable this mpi hook is to select GPUs
+                    # If we have more than 1 CUDA GPU available, use MPI rank to select it
+                    # We cannot assume that the rank is sequential on each node, so we need
+                    # to determine a sequential rank on each node.
+                    # This assumes that all MPI tasks on each node see the same devices.
+                    nodes = mpi.gather(platform.node(), root=0)
+                    nodes_ct = {}
+                    local_ranks = []
+                    if mpi.Get_rank() == 0:
+                        for n in nodes:
+                            if n in nodes_ct:
+                                nodes_ct[n] += 1
+                            else:
+                                nodes_ct[n] = 0
+                            local_ranks.append(nodes_ct[n])
+                    r = mpi.scatter(local_ranks, root=0)
+                    # Before assigning the GPU, sort the found devices by their PCI id, because the device
+                    # order can be different on two process on the same node (!)
+                    benchmark_results = [(r[0], r[1], r[0].pci_bus_id()) for r in benchmark_results]
+                    benchmark_results = list(sorted(benchmark_results, key=lambda t: t[2]))
+                    if verbose:
+                        print("select_gpu using MPI: node=%s mpi_rank=%d, using GPU #%d/%d PCI:" %
+                              (platform.node(), mpi.Get_rank(), r % nb, nb), benchmark_results[r % nb][0].pci_bus_id())
+                return self.set_device(benchmark_results[r % nb][0], test_fft=False, verbose=verbose)
+            else:
+                return self.set_device(benchmark_results[gpu_rank][0], test_fft=False, verbose=verbose)
 
         test_opencl = False
         if language is None:
@@ -308,7 +352,7 @@ class ProcessingUnit(object):
                     fft_axis = None
                 benchmark_results += opencl_device.available_gpu_speed(fft_shape=fft_shape, axes=fft_axis,
                                                                        min_gpu_mem=None, verbose=verbose,
-                                                                       gpu_name=gpu_name)
+                                                                       gpu_name=gpu_name, ranking=ranking)
             if language is not None:
                 if 'opencl' in language.lower():
                     if len(benchmark_results) == 0:
@@ -331,9 +375,33 @@ class ProcessingUnit(object):
                 # This requires that a language be selected, or each device will be listed twice for cuda+opencl
                 nb = len(benchmark_results)
                 r = mpi.Get_rank()
+                if nb > 1:
+                    # TODO: evaluate how reliable this mpi hook is to select GPUs
+                    # If we have more than 1 CUDA GPU available, use MPI rank to select it
+                    # We cannot assume that the rank is sequential on each node, so we need
+                    # to determine a sequential rank on each node.
+                    # This assumes that all MPI tasks on each node see the same devices.
+                    nodes = mpi.gather(platform.node(), root=0)
+                    nodes_ct = {}
+                    local_ranks = []
+                    if mpi.Get_rank() == 0:
+                        for n in nodes:
+                            if n in nodes_ct:
+                                nodes_ct[n] += 1
+                            else:
+                                nodes_ct[n] = 0
+                            local_ranks.append(nodes_ct[n])
+                    r = mpi.scatter(local_ranks, root=0)
+                    # Before assigning the GPU, sort the found devices by their address, because the device
+                    # order can be different on two process on the same node (is that also true for opencl?)
+                    benchmark_results = [(r[0], r[1], r[0].int_ptr) for r in benchmark_results]
+                    benchmark_results = list(sorted(benchmark_results, key=lambda t: t[2]))
+                    if verbose:
+                        print("select_gpu using MPI: node=%s mpi_rank=%d, using GPU #%d/%d ptr:" %
+                              (platform.node(), mpi.Get_rank(), r % nb, nb), benchmark_results[r % nb][0].int_ptr)
                 return self.set_device(benchmark_results[gpu_rank][r % nb], test_fft=False, verbose=verbose)
 
-            if ranking == 'fft_speed':
+            if ranking in ['fft', 'bandwidth']:
                 benchmark_results = sorted(benchmark_results, key=lambda t: -t[1])
             if len(benchmark_results) <= gpu_rank:
                 if gpu_name is not None:
@@ -382,39 +450,222 @@ class ProcessingUnit(object):
         warnings.warn("ProcessingUnit.synchronize() is deprecated. Use finish() instead.", DeprecationWarning)
         return self.finish()
 
-    def fft(self, d, axes=None, normalize=False):
+    def fft(self, src, dest, ndim=None, norm=False, return_scale=True, **kwargs):
         """
-        Perform an out-of-place FFT transform.
+        Perform a FFT transform, in or out-of-place. This will automatically
+        create a FFT plan as needed using the appropriate backend.
+        This supports both C2C and R2C transforms according to the type of
+        the src and dest arrays.
+        For R2C, if the last dimension of the source is n, the last dimension
+        of the half-Hermitian result transform is n//2+1, when out-of-place.
+        If an inplace R2C transform is used (only using VkFFT backend), the
+        source array must have the last dimension padded by two values, which are
+        ignored in the source, and used to keep the same number of bytes in the output.
 
-        This function is mostly a placeholder for derived GPU implementations, which could be useful for
-        occasional FT on large arrays.
-        :param d: the array to be transformed
-        :param normalize: if True, the FFT will keep the norm of the array (abs(d)**2).sum()
-        :param axes: the transform axes
-        :return: the result of the FFT transform
+        :param src: the source array
+        :param dest: the destination array. can be the same as src for an in-place transform.
+        :param ndim: number of dimensions for the transform, the FFT is done only for
+            the last ndim axes. If None, transform applies to all dimensions.
+        :param norm: if True, the FFT will keep the L2 norm of the array (abs(d)**2).sum(),
+            like the norm='ortho' of scipy and numpy.
+        :param return_scale: if True, returns the scale factor by which the dest array must
+            be multiplied so that the L2 norm is the same as the src array.
+        :param kwargs: some extra parameters can be given depending on the backend:
+            nproc=4: number of parallel process (CPU only, will default to available CPU)
+            stream=s: specifying a CUDA stream for the transform
+        :return: nothing, or the scale to keep the L2 norm if return_scale is True.
         """
-        # TODO: derive and accelerate this function for different processing units
-        d = fftn(d, axes=axes)
-        if normalize:
-            d /= np.sqrt(d.size)
-        return d
+        if 'nproc' in kwargs:
+            nproc = kwargs['nproc']
+        else:
+            nproc = self.nproc
+        if ndim is None:
+            ax = tuple(list(range(src.ndim)))
+        else:
+            ax = tuple(list(range(src.ndim))[-ndim:])
+        if np.iscomplexobj(src) and np.iscomplexobj(dest):
+            assert src.shape == dest.shape
+            if norm:
+                dest[:] = fftn(src, axes=ax, workers=nproc, norm="ortho")
+            else:
+                dest[:] = fftn(src, axes=ax, workers=nproc)
+        elif np.isrealobj(src) and np.iscomplexobj(dest):
+            assert src.shape[:-1] == dest.shape[:-1]
+            assert src.shape[-1] // 2 + 1 == dest.shape[-1]
+            if norm:
+                dest[:] = rfftn(src, axes=ax, workers=nproc, norm="ortho")
+            else:
+                dest[:] = rfftn(src, axes=ax, workers=nproc)
+        else:
+            raise ProcessingUnitException("ProcessingUnit.fft(): must be a C2C or R2C transform")
+        if return_scale:
+            if norm:
+                return 1
+            else:
+                return self.fft_scale(src, ndim=ndim)[0]
 
-    def ifft(self, d, axes=None, normalize=False):
+    def ifft(self, src, dest, ndim=None, norm=False, return_scale=True, **kwargs):
         """
-        Perform an out-of-place FFT transform.
+        Perform a FFT transform, in or out-of-place. This will automatically
+        create a FFT plan as needed using the appropriate backend.
+        This supports both C2C and C2R transforms according to the type of
+        the src and dest arrays.
+        For C2R, if the last dimension of the destination is n, the last dimension
+        of the half-Hermitian source is n//2+1, when out-of-place.
+        If an inplace C2R transform is used (only using VkFFT backend), the
+        destination array will have the last dimension padded by two values,
+        to keep the same number of bytes as in the source.
 
-        This function is mostly a placeholder for derived GPU implementations, which could be useful for
-        occasional FT on large arrays.
-        :param d: the array to be transformed
-        :param normalize: if True, the FFT will keep the norm of the array (abs(d)**2).sum()
-        :param axes: the transform axes
-        :return: the result of the inverse FFT transform
+        :param src: the source array
+        :param dest: the destination array. can be the same as src for an in-place transform.
+        :param ndim: number of dimensions for the transform, the FFT is done only for
+            the last ndim axes. If None, transform applies to all dimensions.
+        :param norm: if True, the FFT will keep the L2 norm of the array (abs(d)**2).sum(),
+            like the norm='ortho' of scipy and numpy.
+        :param return_scale: if True, returns the scale factor by which the dest array must
+            be multiplied so that the L2 norm is the same as the src array.
+        :param kwargs: some extra parameters can be given depending on the backend:
+            nproc=4: number of parallel process (CPU only, will default to available CPU)
+            stream=s: specifying a CUDA stream for the transform
+        :return: nothing, or the scale to keep the L2 norm if return_scale is True.
         """
-        # TODO: derive and accelerate this function for different processing units
-        d = fftn(d, axes=axes)
-        if normalize:
-            d *= np.sqrt(d.size)
-        return d
+        if 'nproc' in kwargs:
+            nproc = kwargs['nproc']
+        else:
+            nproc = self.nproc
+        if ndim is None:
+            ax = tuple(list(range(src.ndim)))
+        else:
+            ax = tuple(list(range(src.ndim))[-ndim:])
+        if np.iscomplexobj(src) and np.iscomplexobj(dest):
+            assert src.shape == dest.shape
+            if norm:
+                dest[:] = ifftn(src, axes=ax, workers=nproc, norm="ortho")
+            else:
+                dest[:] = ifftn(src, axes=ax, workers=nproc)
+        elif np.iscomplexobj(src) and np.isrealobj(dest):
+            assert src.shape[:-1] == dest.shape[:-1]
+            assert dest.shape[-1] // 2 + 1 == src.shape[-1]
+            if norm:
+                dest[:] = irfftn(src, axes=ax, workers=nproc, norm="ortho")
+            else:
+                dest[:] = irfftn(src, axes=ax, workers=nproc)
+        else:
+            raise ProcessingUnitException("ProcessingUnit.ifft(): must be a C2C or C2R transform")
+        if return_scale:
+            if norm:
+                return 1
+            else:
+                return self.fft_scale(dest, ndim=ndim)[1]
+
+    def fft_scale(self, src, ndim=None, axes=None):
+        """
+        Get the forward and backward FFT scales by which the destination array
+        must be multiplied to keep the L2 norm of the input array.
+        :param src: the source (for the forward transform) array or its shape.
+            For a R2C/C2R transform, this must be the real array.
+        :param ndim: number of dimensions for the transform, the FFT is done only for
+            the last ndim axes. If None, transform applies to all dimensions.
+        :param axes: reserved for future use
+        :return: a tuple with the forward and backward scales
+        """
+        if isinstance(src, tuple) or isinstance(src, list):
+            sh = src
+        else:
+            sh = src.shape
+        if ndim is None:
+            s = np.sqrt(np.prod(sh))
+        else:
+            s = np.sqrt(np.prod(sh[-ndim:]))
+        return 1 / s, s
+
+    def free_fft_plans(self):
+        """
+        If memory was allocated for FFT plans, free it if possible
+        :return: nothing
+        """
+        pass
+
+    def fft_benchmark(self, shape, dtype, ndim, inplace=False, nb=None, nb_test=3, free_plan=True):
+        """
+        Benchmark a couple of FT and IFT transform
+        :param shape: shape of the benchmarked transform
+        :param dtype: the dtype for the transform. If it is real, a R2C/C2R transform is performed
+        :param ndim: number of dimensions for the transform
+        :param inplace: if True, an inplace transform is used
+        :param nb: the number of couple of transforms used for testing. If None, this
+            will be automatically selected for accuracy.
+        :param nb_test: number of tests performed, the best result is returned
+        :param free_plan: if True, free plans at the end of the test
+        :return: a tuple (dt, throughput) with dt the time for a couple of FT+IFT transforms,
+            throughput is the memory throughput computed by assuming that each transform axis
+            uses exactly one read and one write. dt in seconds, throughput in Gbytes/s
+        """
+        a = np.zeros(shape, dtype=dtype)
+        a.fill(2.0)
+        if dtype in [np.complex64, np.complex128]:
+            if inplace:
+                b = np.empty_like(a)
+            else:
+                b = a
+        self.fft(a, b, ndim=ndim, norm=False)
+        self.finish()
+        # print(100 * 1024 ** 3 / (ndim * 2 * 2 * a.nbytes))
+        if nb is None:
+            # Target about 0.1s test, minimum 10GB/s
+            nb = 0.1 * 1 * 1024 ** 3 / (ndim * 2 * 2 * a.nbytes)
+            nb = max(int(nb), 1)
+            nb = min(nb, 1000)
+        dt_best = 0
+        for i in range(nb_test):
+            t0 = timeit.default_timer()
+            for i in range(nb):
+                self.fft(a, b, ndim=ndim, norm=False)
+                self.fft(b, a, ndim=ndim, norm=False)
+            self.finish()
+            dt = timeit.default_timer() - t0
+            if dt_best == 0 or dt < dt_best:
+                dt_best = dt
+        dt_best /= nb
+        thr = ndim * 2 * 2 * a.nbytes / dt_best / 1024 ** 3
+        if free_plan:
+            self.free_fft_plans()
+        return dt_best, thr
+
+    def max_prime_fft_radix(self):
+        """Return the largest prime number accepted for radix-based FFT transforms.
+        Larger prime numbers in the array size decomposition may be possible but
+        using a slower Bluestein algorithm"""
+        return 13
+
+    @abstractmethod
+    def pagelocked_empty(self, shape, dtype):
+        """
+        Allocate a numpy array in pinned / Page-locked memory.
+        :param shape: the array shape
+        :param dtype: the array dtype
+        :return: the page-locked numpy array
+        """
+        pass
+
+    def pagelocked_empty_like(self, arr):
+        """
+        Allocate a numpy array in pinned / Page-locked memory, with
+        the same shape and dtype as another one.
+        :param arr: the array
+        :return: the page-locked numpy array
+        """
+        return self.pagelocked_empty(arr.shape, arr.dtype)
+
+    def to_pagelocked(self, arr):
+        """
+        Convert a numpy array to a new one with pinned / Page-locked memory.
+        :param arr: the array to convert
+        :return: the page-locked numpy array
+        """
+        a = self.pagelocked_empty_like(arr)
+        a[:] = arr
+        return a
 
 
 # The default processing unit
