@@ -45,7 +45,7 @@ data = BraggPtychoData(iobs=np.empty((nb, npsi, ny, nx), dtype=np.float32), posi
 # Import existing probe from 2D ptycho
 d = np.load("/Users/favre/Analyse/201606id01-FZP/ResultsScan0013/latest.npz")
 # d = np.load("/Users/vincent/Analyse/201606id01-FZP/ResultsScan0000/latest.npz")
-pr = Wavefront(d=np.fft.fftshift(d['probe']), z=0, pixel_size=d['pixelsize'], wavelength=wavelength)
+pr = Wavefront(d=fftshift(d['probe']), z=0, pixel_size=d['pixelsize'], wavelength=wavelength)
 
 # Create main Bragg Ptycho object
 p = BraggPtycho(probe=pr, data=data, support=None)
