@@ -13,12 +13,13 @@ import time
 from sys import stdout
 import warnings
 import numpy as np
-from scipy.fftpack import fftn, ifftn, fftshift, fftfreq
+from scipy.fft import fftn, ifftn, fftshift, fftfreq
 from scipy.spatial import cKDTree
 from scipy.interpolate import griddata
 from scipy.ndimage.measurements import center_of_mass
 from matplotlib.backends.backend_agg import FigureCanvasAgg
 from matplotlib.figure import Figure
+from matplotlib.colors import LogNorm
 from pynx.utils.matplotlib import pyplot as plt
 from .ptycho import Ptycho, OperatorPtycho, algo_string
 from ..utils.plot_utils import show_obj_probe, complex2rgbalin, colorwheel
@@ -64,6 +65,36 @@ patch_method(Ptycho)
 #################################################################################################################
 
 
+def calc_quadratic_phase_factor(p: Ptycho, factor):
+    """
+    Calculates the quadratic phase factor and stores it in the Ptycho object.
+    Two types can be calculated, for far and near field propagation.
+    In the near field case, the factor does not need to be multiplied by
+    the array sizes, and is not fft-shifted.
+
+    :param p: the Ptycho object for wich the quadratic phase factor is computed
+    :param factor:
+    :return: Nothing
+    """
+    ny = np.int32(p._probe.shape[-2])
+    nx = np.int32(p._probe.shape[-1])
+    need_calc_q = True
+    if has_attr_not_none(p, '_cpu_tmp_quad_phase'):
+        if np.isclose(p._cpu_tmp_quad_phase[1], abs(factor)):
+            need_calc_q = False
+
+    if need_calc_q:
+        if p.data.near_field:
+            x = fftfreq(nx)
+            y = fftfreq(ny)[:, np.newaxis]
+            q = np.exp(1j * abs(factor) * (x * x + y * y))
+        else:
+            x = fftfreq(nx) * nx
+            y = fftfreq(ny)[:, np.newaxis] * ny
+            q = fftshift(np.exp(1j * abs(factor) * (x * x + y * y)))
+        p._cpu_tmp_quad_phase = (q.astype(np.complex64), abs(factor))
+
+
 class CPUOperatorPtycho(OperatorPtycho):
     """
     Base class for a operators on CDI objects using CPU
@@ -117,26 +148,6 @@ class CPUOperatorPtycho(OperatorPtycho):
         if has_attr_not_none(p, '_cpu_view') is False:
             p._cpu_view = {}
 
-    def calc_quadratic_phase_factor(self, p: Ptycho, factor):
-        """
-        Calculates the quadratic phase factor and stores it in the Ptycho object
-        :param factor:
-        :return: Nothing
-        """
-        ny = np.int32(p._probe.shape[-2])
-        nx = np.int32(p._probe.shape[-1])
-        need_calc_q = True
-        if has_attr_not_none(p, '_cpu_tmp_quad_phase'):
-            if np.isclose(p._cpu_tmp_quad_phase[1], abs(factor)):
-                need_calc_q = False
-
-        if need_calc_q:
-            x = fftfreq(nx) * nx
-            y = fftfreq(ny)[:, np.newaxis] * ny
-
-            q = fftshift(np.exp(1j * abs(factor) * (x * x + y * y)))
-            p._cpu_tmp_quad_phase = (q.astype(np.complex64), abs(factor))
-
     def timestamp_increment(self, p):
         p._timestamp_counter += 1
         p._cpu_timestamp_counter = p._timestamp_counter
@@ -230,8 +241,7 @@ class CPUOperatorPtychoSum(OperatorSum, CPUOperatorPtycho):
         if isinstance(op1, CPUOperatorPtycho) is False or isinstance(op2, CPUOperatorPtycho) is False:
             raise OperatorException(
                 "ERROR: cannot add a CLOperatorCDI with a non-CLOperatorCDI: %s + %s" % (str(op1), str(op2)))
-        # We can only have a sum of two CLOperatorCDI, so they must have a processing_unit attribute.
-        CPUOperatorPtycho.__init__(self, op1.processing_unit)
+        CPUOperatorPtycho.__init__(self)
         OperatorSum.__init__(self, op1, op2)
 
         # We need to cherry-pick some functions & attributes doubly inherited
@@ -249,7 +259,7 @@ class CPUOperatorPtychoSum(OperatorSum, CPUOperatorPtycho):
 # The only purpose of this class is to make sure it inherits from CLOperatorCDI and has a processing unit
 class CPUOperatorPtychoPower(OperatorPower, CPUOperatorPtycho):
     def __init__(self, op, n):
-        CPUOperatorPtycho.__init__(self, op.processing_unit)
+        CPUOperatorPtycho.__init__(self)
         OperatorPower.__init__(self, op, n)
 
         # We need to cherry-pick some functions & attributes doubly inherited
@@ -296,7 +306,7 @@ class ShowObjProbe(CPUOperatorPtycho):
         if p.data.near_field:
             show_obj_probe(p.get_obj(), p.get_probe(), stit=self.title, fig_num=self.fig_num,
                            pixel_size_object=p.pixel_size_object, scan_area_obj=None, scan_area_probe=None,
-                           scan_pos=p.get_scan_area_points())
+                           scan_pos=p.get_scan_area_points(), invert_yaxis=True)
         else:
             obj = p.get_obj(remove_obj_phase_ramp=self.remove_obj_phase_ramp)
 
@@ -311,6 +321,49 @@ class ShowObjProbe(CPUOperatorPtycho):
         pass
 
 
+class ShowObsCalc(CPUOperatorPtycho):
+    """
+    Class to display calculated and observed frames.
+
+    Note that this operator will change the Psi array currently
+    in memory. Therefore this operator should not be used during
+    algorithms (e.g. DM) where the previous Psi value is used.
+    """
+
+    def __init__(self, i: int, fig_num=-1, title=None):
+        """
+
+        :param i: the index of the frame to display
+        :param fig_num: the matplotlib figure number. if None, a new figure will be created.
+            If -1 (the default), the current figure will be used.
+        :param title: the title for the figure. Default will be the frame number.
+        """
+        super().__init__()
+        self.i = i
+        self.fig_num = fig_num
+        self.title = title
+
+    def op(self, p: Ptycho):
+        icalc = get_icalc(p, self.i, shift=True)
+        iobs = p.get_iobs(self.i, shift=True)
+        if self.fig_num != -1:
+            fig = plt.figure(self.fig_num)
+        plt.subplot(121)
+        plt.imshow(iobs, norm=LogNorm())
+        plt.colorbar()
+        plt.title("Obs")
+        plt.subplot(122)
+        plt.imshow(icalc, norm=LogNorm())
+        plt.colorbar()
+        plt.title("Calc")
+        plt.suptitle(self.title or "#%d" % self.i)
+        return p
+
+    def timestamp_increment(self, p):
+        # This display operation does not modify the data.
+        pass
+
+
 #################################################################################################################
 ########################  End of Exclusive CPU operators  #######################################################
 #################################################################################################################
@@ -407,8 +460,9 @@ class ObjProbe2Psi(CPUOperatorPtycho):
         nb_obj = np.int32(p._obj.shape[0])
         nyo = np.int32(p._obj.shape[-2])
         nxo = np.int32(p._obj.shape[-1])
-        f = np.float32(np.pi / (p.data.wavelength * p.data.detector_distance))
-        self.calc_quadratic_phase_factor(p, f * p.pixel_size_object ** 2)
+        if not p.data.near_field:
+            f = np.float32(np.pi / (p.data.wavelength * p.data.detector_distance))
+            calc_quadratic_phase_factor(p, f * p.pixel_size_object ** 2)
         obs = p.data.iobs
         n_stack = len(obs)
         p._psi = np.empty((nb_obj, nb_probe, n_stack, ny, nx), dtype=np.complex64)
@@ -416,10 +470,13 @@ class ObjProbe2Psi(CPUOperatorPtycho):
         for iobj in range(len(p._obj)):
             for iprobe in range(len(p._probe)):
                 for i in range(n_stack):
-                    dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
-                    cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                    op = p._probe[iprobe] * p._obj[iobj, cy:cy + ny, cx:cx + nx]
-                    if p.data.near_field is False:
+                    if p.data.posx.mask[i]:
+                        op = p._probe[iprobe].copy()  # TODO: check copy() is necessary. cow?
+                    else:
+                        dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
+                        cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
+                        op = p._probe[iprobe] * p._obj[iobj, cy:cy + ny, cx:cx + nx]
+                    if not p.data.near_field:
                         op *= p._cpu_tmp_quad_phase[0]
                     p._psi[iobj, iprobe, i] = fftshift(op)
         return p
@@ -440,7 +497,7 @@ class FT(CPUOperatorPtycho):
 
     def op(self, pty):
         if self.scale:
-            pty._psi = fftn(pty._psi, axes=(-2, -1)) / np.sqrt(pty._psi.shape[-2] * pty._psi.shape[-1])
+            pty._psi = fftn(pty._psi, axes=(-2, -1), norm="ortho")
         else:
             pty._psi = fftn(pty._psi, axes=(-2, -1))
         return pty
@@ -461,7 +518,7 @@ class IFT(CPUOperatorPtycho):
 
     def op(self, pty):
         if self.scale:
-            pty._psi = ifftn(pty._psi, axes=(-2, -1)) * np.sqrt(pty._psi.shape[-2] * pty._psi.shape[-1])
+            pty._psi = ifftn(pty._psi, axes=(-2, -1), norm="ortho")
         else:
             pty._psi = ifftn(pty._psi, axes=(-2, -1))
         return pty
@@ -488,11 +545,11 @@ class QuadraticPhase(CPUOperatorPtycho):
         self.factor = np.float32(factor)
 
     def op(self, p):
-        self.calc_quadratic_phase_factor(p, self.factor)
+        calc_quadratic_phase_factor(p, self.factor)
         if self.factor > 0:
-            p._psi *= self.scale * p.cpu_tmp_quad[0]
+            p._psi *= self.scale * p._cpu_tmp_quad_phase[0]
         else:
-            p._psi *= self.scale / p.cpu_tmp_quad[0]
+            p._psi *= self.scale / p._cpu_tmp_quad_phase[0]
         return p
 
 
@@ -558,8 +615,13 @@ class Calc2Obs(CPUOperatorPtycho):
         self.nb_photons_per_frame = nb_photons_per_frame
         self.poisson_noise = poisson_noise
 
-    def op(self, p):
-        p.data.iobs = (np.abs(p._psi) ** 2).sum(axis=(0, 1))
+    def op(self, p: Ptycho):
+        if p.data.near_field:
+            prop = PropagateNearField(forward=True)
+        else:
+            prop = FT(scale=True)
+        p = prop * ObjProbe2Psi() * p
+        p.data.iobs = (np.abs(p._psi) ** 2).sum(axis=(0, 1)) + p.get_background()
 
         if self.nb_photons_per_frame is not None:
             p.data.iobs *= self.nb_photons_per_frame / (p.data.iobs.sum() / len(p.data.iobs))
@@ -629,7 +691,9 @@ class LLK(CPUOperatorPtycho):
         # Poisson
         llk = np.zeros(iobs.shape, dtype=np.float32)
         idx = np.where(iobs > 0)
-        llk[idx] = np.take(icalc - iobs + iobs * np.log(iobs / icalc), idx)
+        iobs1 = np.take(iobs, idx)
+        icalc1 = np.take(icalc, idx)
+        llk[idx] = icalc1 - iobs1 + iobs1 * np.log(iobs1 / icalc1)
         idx = np.where(iobs == 0)
         llk[idx] = np.take(icalc, idx)
         p.llk_poisson = llk.sum()
@@ -650,15 +714,19 @@ class Psi2ObjProbe(CPUOperatorPtycho):
     Computes updated Obj(r) and Probe(r-r_j) contributions from Psi.
     """
 
-    def __init__(self, update_object=True, update_probe=False):
+    def __init__(self, update_object=True, update_probe=False, obj_inertia=1e-2, probe_inertia=1e-3,
+                 debug=False):
         """
 
         """
         super(Psi2ObjProbe, self).__init__()
         self.update_object = update_object
         self.update_probe = update_probe
+        self.obj_inertia = obj_inertia
+        self.probe_inertia = probe_inertia
+        self.debug = debug
 
-    def op(self, p):
+    def op(self, p: Ptycho):
         ny = np.int32(p._probe.shape[-2])
         nx = np.int32(p._probe.shape[-1])
         nb_probe = np.int32(p._probe.shape[0])
@@ -666,8 +734,11 @@ class Psi2ObjProbe(CPUOperatorPtycho):
         nyo = np.int32(p._obj.shape[-2])
         nxo = np.int32(p._obj.shape[-1])
         f = np.float32(np.pi / (p.data.wavelength * p.data.detector_distance))
-        self.calc_quadratic_phase_factor(p, f * p.pixel_size_object ** 2)
-        q = p._cpu_tmp_quad_phase[0]
+        if not p.data.near_field:
+            calc_quadratic_phase_factor(p, f * p.pixel_size_object ** 2)
+            q = p._cpu_tmp_quad_phase[0]
+        else:
+            q = np.float32(1)
         obs = p.data.iobs
         n_stack = len(obs)
         px, py = p.data.pixel_size_object()
@@ -678,13 +749,12 @@ class Psi2ObjProbe(CPUOperatorPtycho):
                 for iprobe in range(nb_probe):
                     pr = p._probe[iprobe]
                     for i in range(n_stack):
-                        dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
-                        cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                        obj_new[iobj, cy:cy + ny, cx:cx + nx] += fftshift(p._psi[iobj, iprobe, i] / q) * pr.conj()
-                        obj_norm[cy:cy + ny, cx:cx + nx] += abs(pr) ** 2
-            reg = obj_norm.max() * 1e-2
-
-            p._obj = (obj_new + reg * p._obj) / np.fmax(obj_norm + reg, 1e-12)
+                        if not p.data.posx.mask[i]:
+                            dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
+                            cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
+                            obj_new[iobj, cy:cy + ny, cx:cx + nx] += fftshift(p._psi[iobj, iprobe, i]) * (q * pr).conj()
+                            if iobj == 0:
+                                obj_norm[cy:cy + ny, cx:cx + nx] += abs(pr) ** 2
 
         if self.update_probe:
             probe_norm = np.zeros_like(p._probe[0], dtype=np.float32)
@@ -693,14 +763,31 @@ class Psi2ObjProbe(CPUOperatorPtycho):
             for iobj in range(len(p._obj)):
                 for iprobe in range(len(p._probe)):
                     for i in range(n_stack):
-                        dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
-                        cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                        ob = p._obj[iobj, cy:cy + ny, cx:cx + nx]
-                        p._probe[iprobe] += fftshift(p._psi[iobj, iprobe, i] / q) * ob.conj()
-                        probe_norm += abs(ob) ** 2
-            reg = probe_norm.max() * 1e-2
+                        if p.data.posx.mask[i]:
+                            p._probe[iprobe] += fftshift(p._psi[iobj, iprobe, i]) * q.conj()
+                            probe_norm += 1
+                        else:
+                            dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
+                            cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
+                            ob = p._obj[iobj, cy:cy + ny, cx:cx + nx]
+                            p._probe[iprobe] += fftshift(p._psi[iobj, iprobe, i]) * (q * ob).conj()
+                            if iprobe == 0:
+                                probe_norm += abs(ob) ** 2
+            reg = probe_norm.max() * self.probe_inertia
+            if self.debug:
+                p._probe_new = p._probe.copy()
+                p._probe_norm = probe_norm
             p._probe = (p._probe + reg * probe_old) / np.fmax(probe_norm + reg, 1e-12)
 
+        if self.update_object:
+            # Final object update once the probe has been updated
+            reg = obj_norm.max() * self.obj_inertia
+
+            p._obj = (obj_new + reg * p._obj) / np.fmax(obj_norm + reg, 1e-12)
+            if self.debug:
+                p._obj_new = obj_new
+                p._obj_norm = obj_norm
+
         return p
 
 
@@ -713,7 +800,8 @@ class AP(CPUOperatorPtycho):
     """
 
     def __init__(self, update_object=True, update_probe=False, update_background=False, floating_intensity=False,
-                 nb_cycle=1, calc_llk=False, show_obj_probe=False, fig_num=-1, zero_phase_ramp=True):
+                 nb_cycle=1, calc_llk=False, show_obj_probe=False, fig_num=-1, obj_inertia=0.01,
+                 probe_inertia=0.001, zero_phase_ramp=True, background_smooth_sigma=0):
         """
 
         :param update_object: update object ?
@@ -726,9 +814,13 @@ class AP(CPUOperatorPtycho):
         :param show_obj_probe: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object and probe, as for ShowObjProbe()
+        :param obj_inertia: the updated object retains this relative amount of the previous object.
+        :param probe_inertia: the updated probe retains this relative amount of the previous probe.
         :param zero_phase_ramp: if True, the conjugate phase ramp in the object and probe will be removed
                                 by centring the FT of the probe, at the end and before every display.
                                 Ignored for near field.
+        :param background_smooth_sigma: gaussian convolution parameter for the background update
+            (ignored on CPU)
         """
         super(AP, self).__init__()
         self.update_object = update_object
@@ -739,6 +831,8 @@ class AP(CPUOperatorPtycho):
         self.calc_llk = calc_llk
         self.show_obj_probe = show_obj_probe
         self.fig_num = fig_num
+        self.obj_inertia = obj_inertia
+        self.probe_inertia = probe_inertia
         self.zero_phase_ramp = zero_phase_ramp
 
     def __pow__(self, n):
@@ -751,6 +845,7 @@ class AP(CPUOperatorPtycho):
         return AP(update_object=self.update_object, update_probe=self.update_probe,
                   floating_intensity=self.floating_intensity, nb_cycle=self.nb_cycle * n,
                   calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe, fig_num=self.fig_num,
+                  obj_inertia=self.obj_inertia, probe_inertia=self.probe_inertia,
                   zero_phase_ramp=self.zero_phase_ramp)
 
     def op(self, p: Ptycho):
@@ -763,7 +858,8 @@ class AP(CPUOperatorPtycho):
                 pass  # TODO: update while in Fourier space
             p = PropagateApplyAmplitude(calc_llk=calc_llk) * ObjProbe2Psi() * p
 
-            p = Psi2ObjProbe(update_object=self.update_object, update_probe=self.update_probe) * p
+            p = Psi2ObjProbe(update_object=self.update_object, update_probe=self.update_probe,
+                             obj_inertia=self.obj_inertia, probe_inertia=self.probe_inertia) * p
 
             if calc_llk:
                 p.update_history(mode='llk', update_obj=self.update_object, update_probe=self.update_probe,
@@ -794,31 +890,70 @@ class DM(CPUOperatorPtycho):
     as well as updating the object and/or probe.
     """
 
-    def __init__(self, update_object=True, update_probe=True, nb_cycle=1, calc_llk=False, show_obj_probe=False,
+    def __init__(self, update_object=True, update_probe=True, update_background=False, nb_cycle=1,
+                 calc_llk=False, show_obj_probe=False,
                  fig_num=-1, obj_smooth_sigma=0, obj_inertia=0.01, probe_smooth_sigma=0, probe_inertia=0.001,
-                 center_probe_n=0, center_probe_max_shift=5, loop_obj_probe=1, zero_phase_ramp=True):
+                 center_probe_n=0, center_probe_max_shift=5, loop_obj_probe=1, update_pos=False, pos_mult=1,
+                 pos_max_shift=2, pos_min_shift=0, pos_threshold=0.2, pos_history=False, zero_phase_ramp=True,
+                 background_smooth_sigma=0):
         """
         TODO: Not all parameters are implemented in the CPU DM version.
         :param update_object: update object ?
         :param update_probe: update probe ?
+        :param update_background: update background ? [Ignored on CPU]
         :param nb_cycle: number of cycles to perform. Equivalent to DM(...)**nb_cycle
         :param calc_llk: if True, calculate llk while in Fourier space. If a positive integer is given, llk will be
                          calculated every calc_llk cycle
         :param show_obj_probe: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object and probe, as for ShowObjProbe()
+        :param obj_smooth_sigma: if > 0, the previous object array (used for inertia) will convoluted by a gaussian
+                                 array of this standard deviation. [Ignored on CPU]
+        :param obj_inertia: the updated object retains this relative amount of the previous object.
+        :param probe_smooth_sigma: if > 0, the previous probe array (used for inertia) will convoluted by a gaussian
+                                   array of this standard deviation. [Ignored on CPU]
+        :param probe_inertia: the updated probe retains this relative amount of the previous probe.
+        :param center_probe_n: test the probe every N cycle for deviation from the center. If deviation is larger
+                               than center_probe_max_shift, probe and object are shifted to correct. If 0 (the default),
+                               the probe centering is never calculated. [Ignored on CPU]
+        :param center_probe_max_shift: maximum deviation from the center (in pixels) to trigger a position correction
+            [Ignored on CPU]
+        :param loop_obj_probe: when both object and probe are updated, it can be more stable to loop the object
+                               and probe update for a more stable optimisation, but slower.
+        :param update_pos: positive integer, if >0, update positions every 'update_pos' cycle. Note that object and
+                           probe are not updated during the same cycle as positions. Still experimental, recommended
+                           are 5, 10 (default=False, positions are not updated). [Ignored on CPU]
+        :param pos_max_shift: maximum allowed shift (in pixels) per scan position (default=2) [Ignored on CPU]
+        :param pos_min_shift: minimum required shift (in pixels) per scan position (default=0) [Ignored on CPU]
+        :param pos_threshold: if the integrated grad_obj*probe along dx or dy is
+            smaller than (grad_obj*probe).mean()*threshold, then the shift is ignored.
+            This allows to prevent position shifts where there is little contrast in
+            the object. It applies independently to x and y. [Ignored on CPU]
+        :param pos_mult: multiply the calculated position shifts by this value. Useful since the calculated
+                                shifts usually are a fraction of the actual shift. [Ignored on CPU]
+        :param pos_history: if True, save the position history (for debugging) [Ignored on CPU]
         :param zero_phase_ramp: if True, the conjugate phase ramp in the object and probe will be removed
                                 by centring the FT of the probe, at the end and before every display.
                                 Ignored for near field.
+        :param background_smooth_sigma: gaussian convolution parameter for the background update
         """
         super(DM, self).__init__()
         self.nb_cycle = nb_cycle
         self.update_object = update_object
         self.update_probe = update_probe
+        self.update_background = update_background
         self.calc_llk = calc_llk
         self.show_obj_probe = show_obj_probe
         self.fig_num = fig_num
+        self.obj_inertia = obj_inertia
+        self.obj_smooth_sigma = obj_smooth_sigma
+        self.probe_inertia = probe_inertia
+        self.probe_smooth_sigma = probe_smooth_sigma
+        self.center_probe_n = center_probe_n
+        self.center_probe_max_shift = center_probe_max_shift
+        self.loop_obj_probe = loop_obj_probe
         self.zero_phase_ramp = zero_phase_ramp
+        self.background_smooth_sigma = background_smooth_sigma
 
     def __pow__(self, n):
         """
@@ -827,9 +962,14 @@ class DM(CPUOperatorPtycho):
         :return: a new DM operator with the number of cycles multiplied by n
         """
         assert isinstance(n, int) or isinstance(n, np.integer)
-        return DM(update_object=self.update_object, update_probe=self.update_probe, nb_cycle=self.nb_cycle * n,
+        return DM(update_object=self.update_object, update_probe=self.update_probe,
+                  update_background=self.update_background, nb_cycle=self.nb_cycle * n,
                   calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe, fig_num=self.fig_num,
-                  zero_phase_ramp=self.zero_phase_ramp)
+                  obj_smooth_sigma=self.obj_smooth_sigma, obj_inertia=self.obj_inertia,
+                  probe_smooth_sigma=self.probe_smooth_sigma, probe_inertia=self.probe_inertia,
+                  center_probe_n=self.center_probe_n, center_probe_max_shift=self.center_probe_max_shift,
+                  loop_obj_probe=self.loop_obj_probe, zero_phase_ramp=self.zero_phase_ramp,
+                  background_smooth_sigma=self.background_smooth_sigma)
 
     def op(self, p):
         # Calculate starting Psi
@@ -852,7 +992,13 @@ class DM(CPUOperatorPtycho):
             # Psi(n+1) = Psi(n) - P*O + Psi_fourier
             p._psi += psi1 - psi0
 
-            p = Psi2ObjProbe(update_object=self.update_object, update_probe=self.update_probe) * p
+            # For DM we first update object, then the probe
+            # p = Psi2ObjProbe(update_object=self.update_object, update_probe=self.update_probe) * p
+            for i in range(self.loop_obj_probe):
+                if self.update_object:
+                    p = Psi2ObjProbe(update_object=True, update_probe=False, obj_inertia=self.obj_inertia) * p
+                if self.update_probe:
+                    p = Psi2ObjProbe(update_object=False, update_probe=True, probe_inertia=self.probe_inertia) * p
 
             if calc_llk:
                 psi1 = p._psi.copy()
@@ -991,21 +1137,25 @@ class ML(CPUOperatorPtycho):
                     for iprobe in range(nb_probe):
                         pr = p._probe[iprobe]
                         for i in range(n_stack):
-                            dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
-                            cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                            obj_grad[iobj, cy:cy + ny, cx:cx + nx] += fftshift(
-                                p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0]) * pr.conj()
+                            if not p.data.posx.mask[i]:
+                                dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
+                                cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
+                                obj_grad[iobj, cy:cy + ny, cx:cx + nx] += fftshift(
+                                    p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0]) * pr.conj()
 
             if self.update_probe:
                 probe_grad[:] = 0
                 for iobj in range(len(p._obj)):
                     for iprobe in range(len(p._probe)):
                         for i in range(n_stack):
-                            dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
-                            cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
-                            ob = p._obj[iobj, cy:cy + ny, cx:cx + nx]
-                            probe_grad[iprobe] += fftshift(
-                                p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0]) * ob.conj()
+                            if p.data.posx.mask[i]:
+                                probe_grad[iprobe] += fftshift(p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0])
+                            else:
+                                dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
+                                cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy)
+                                ob = p._obj[iobj, cy:cy + ny, cx:cx + nx]
+                                probe_grad[iprobe] += fftshift(
+                                    p._psi[iobj, iprobe, i] / p._cpu_tmp_quad_phase[0]) * ob.conj()
             if False:
                 # TODO: debug only
                 if self.update_object:
@@ -1238,7 +1388,8 @@ class ZeroPhaseRamp(CPUOperatorPtycho):
             return p
         ny, nx = np.int32(p._probe.shape[1]), np.int32(p._probe.shape[2])
 
-        cyx = center_of_mass((abs(fftshift(fftn(fftshift(p._probe)))) ** 2).sum(axis=0))
+        cyx = center_of_mass((abs(fftshift(fftn(fftshift(p._probe, axes=(-1, -2)), axes=(-1, -2)),
+                                           axes=(-1, -2))) ** 2).sum(axis=0))
         dx, dy = cyx[1] - nx / 2, cyx[0] - ny / 2
         # print("ZeroPhaseRamp(): (dx, dy)[probe] = (%6.3f, %6.3f)" % (dx, dy))
 
@@ -1335,7 +1486,8 @@ class AnalyseProbe(CPUOperatorPtycho):
                     try:
                         plt.draw()
                         plt.gcf().canvas.draw()
-                        plt.pause(.001)
+                        if 'ipympl' not in plt.get_backend():  # This outputs the graphs again
+                            plt.pause(.001)
                     except:
                         pass
 
@@ -1378,7 +1530,8 @@ class AnalyseProbe(CPUOperatorPtycho):
                     try:
                         plt.draw()
                         plt.gcf().canvas.draw()
-                        plt.pause(.001)
+                        if 'ipympl' not in plt.get_backend():  # This outputs the graphs again
+                            plt.pause(.001)
                     except:
                         pass
         return p
@@ -1415,9 +1568,10 @@ class PlotPositions(CPUOperatorPtycho):
         dx, dy = x - x0, y - y0
         r0 = np.sqrt(((x.max()) - x.min()) ** 2 + ((y.max()) - y.min()) ** 2)
 
-        if np.allclose(dx / r0, 0) and np.allclose(dy / r0, 0):
+        px, py = p.data.pixel_size_object()
+        if np.allclose(dx / px, 0, atol=0.05) and np.allclose(dy / py, 0, atol=0.05):
             if self.verbose:
-                print("Average shift in positions is too small, not plotting")
+                print("Max shift in positions is too small (<0.05 pixel), not plotting")
             return p
 
         s = np.log10(r0)
@@ -1501,7 +1655,8 @@ class PlotPositions(CPUOperatorPtycho):
             try:
                 plt.draw()
                 plt.gcf().canvas.draw()
-                plt.pause(.001)
+                if 'ipympl' not in plt.get_backend():  # This outputs the graphs again
+                    plt.pause(.001)
             except:
                 pass
 
@@ -1517,12 +1672,20 @@ class CalcIllumination(CPUOperatorPtycho):
     """
 
     def op(self, p: Ptycho):
-        pr = p._probe
+        pr = p._probe.copy()
         nprobe, ny, nx = pr.shape
+        if p.data.padding:
+            padding = p.data.padding
+            pr[:, :padding] = 0
+            pr[:, ny - padding:] = 0
+            pr[:, :, :padding] = 0
+            pr[:, :, nx - padding:] = 0
         nyo, nxo = p._obj.shape[-2:]
         px, py = p.data.pixel_size_object()
         obj_illumination = np.zeros_like(p._obj[0], dtype=np.float32)
         for i in range(len(p.data.iobs)):
+            if p.data.posx.mask[i]:
+                continue
             dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
             cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy, integer=True)
             obj_illumination[cy:cy + ny, cx:cx + nx] += (abs(pr) ** 2).sum(axis=0)
@@ -1555,3 +1718,25 @@ class LoopStack(OperatorPtycho):
 
     def __new__(cls, op):
         return op
+
+
+def get_icalc(p: Ptycho, i, shift=False):
+    """
+    Get the calculated intensity for frame i and a given Ptycho object.
+    Note that this will reset the current psi, so should not be used during
+    algorithms like DM which rely on the previous Psi value.
+
+    :param p: the Ptycho object
+    :param i: the index of the desired frame
+    :param shift: if True, the array will be shifted so that the array is centered
+        on the frame center (and not the corner).
+    :return: the numpy array of the calculated intensity
+    """
+    if p.data.near_field:
+        p = PropagateNearField(forward=True) * ObjProbe2Psi() * p
+    else:
+        p = FT(scale=True) * ObjProbe2Psi() * p
+    icalc = (abs(p._psi[:, :, i]) ** 2).sum(axis=(0, 1)) + p.get_background()
+    if shift:
+        return fftshift(icalc, axes=(-1, -2))
+    return (icalc)
