@@ -13,12 +13,8 @@ from scipy.spatial import ConvexHull
 from scipy import optimize
 from skimage.draw import polygon
 
-# TODO: switch to skimage.feature.phase_cross_correlation before skimage 0.19 is out
-try:
-    from skimage.registration import phase_cross_correlation as register_translation
-except ImportError:
-    from skimage.feature import register_translation
-from ..ptycho import Ptycho as PtychoBase, save_obj_probe_cxi, algo_string
+from ...utils.registration import phase_cross_correlation
+from ..ptycho import Ptycho as PtychoBase, save_obj_probe_cxi, algo_string, phase
 from .operator import *
 from ...mpi import *
 
@@ -68,7 +64,7 @@ class PtychoSplit(PtychoBase):
 
     def save_obj_probe_cxi(self, filename, sample_name=None, experiment_id=None, instrument=None, note=None,
                            process=None, append=False, shift_phase_zero=False, params=None,
-                           remove_obj_phase_ramp=False):
+                           remove_obj_phase_ramp=False, cxi_data='object_probe'):
         self.stitch()  # Will be done only if necessary
         # Gather the different object parts for output
         # Avoid gather since large objects can lead to error like:
@@ -127,6 +123,20 @@ class PtychoSplit(PtychoBase):
                                fftshift(fftfreq(nxo, d=nx / nxo)).astype(np.float32), indexing='ij')
             obj = obj * np.exp(-2j * np.pi * (x * self.mpi_phase_ramp_dx + y * self.mpi_phase_ramp_dy))
 
+        obj_illum = self.mpi_illum_norm
+        if 'object' not in cxi_data:
+            obj = None
+            obj_illum = None
+        elif cxi_data == 'object_phase':
+            if shift_phase_zero:
+                # Get the object and center the phase around 0
+                obj = phase.shift_phase_zero(obj, percent=2, origin=0, mask=self.get_scan_area_obj())
+                shift_phase_zero = False
+            obj = np.angle(obj).astype(np.float16)
+            obj_illum = None
+        if 'probe' not in cxi_data:
+            probe = None
+
         save_obj_probe_cxi(filename, obj, probe, self.data.wavelength,
                            self.data.detector_distance, self.data.pixel_size_detector, llk['poisson'],
                            llk['gaussian'], llk['euclidian'], llk['nb_photons_calc'],
@@ -135,7 +145,7 @@ class PtychoSplit(PtychoBase):
                            scan_area_obj=self.get_mpi_scan_area_obj(), scan_area_probe=self.get_scan_area_probe(),
                            background=self._background, sample_name=sample_name, experiment_id=experiment_id,
                            instrument=instrument, note=note, process=process, append=append,
-                           shift_phase_zero=shift_phase_zero, params=params, obj_illumination=self.mpi_illum_norm,
+                           shift_phase_zero=shift_phase_zero, params=params, obj_illumination=obj_illum,
                            extra_data=extra_data, obj_phase_ramp=(self.mpi_phase_ramp_dx, self.mpi_phase_ramp_dy))
 
     def init_mpi(self):
@@ -273,7 +283,7 @@ class PtychoSplit(PtychoBase):
         # Object
 
         rr, cc = polygon(vy, vx, self.mpi_obj.shape[-2:])
-        self._mpi_scan_area_obj = np.zeros(self.mpi_obj.shape[-2:], dtype=np.bool)
+        self._mpi_scan_area_obj = np.zeros(self.mpi_obj.shape[-2:], dtype=bool)
         self._mpi_scan_area_obj[rr, cc] = True
 
         # scan_area_points are relative to the center of the object
@@ -365,7 +375,7 @@ class PtychoSplit(PtychoBase):
             v1 = np.empty_like(v)
             for i in range(1, self.mpi_size):
                 self.mpic.Recv(v1, source=i, tag=200)
-                r = register_translation(v, v1, upsample_factor=1, space='real')
+                r = phase_cross_correlation(v, v1, upsample_factor=1, space='real')
                 self.mpic.send(r, dest=i, tag=201)
         else:
             self.mpic.Send((abs(self._probe) ** 2).sum(axis=0), dest=0, tag=200)
@@ -451,7 +461,7 @@ class PtychoSplit(PtychoBase):
             dx, dy = self.mpic.recv(source=0, tag=210)
             # print("Got pos shift    (MPI #%02d): (%8.2f, %8.2f) nm" % (self.mpi_rank, dx * 1e9, dy * 1e9))
             px, py = self.data.pixel_size_object()
-            dix, diy = np.int(np.round(dx / px)), np.int(np.round(dy / py))
+            dix, diy = int(np.round(dx / px)), int(np.round(dy / py))
             if (abs(dix) + abs(diy)) > 0:
                 self._obj = np.roll(self._obj, (dix, diy), axis=(-1, -2))
                 # TODO: Should we round the pos shifts at pixel resolution ?
