@@ -15,10 +15,6 @@ import locale
 import timeit
 import warnings
 
-try:
-    import hdf5plugin
-except:
-    print("WARNING: hdf5plugin not found. This may prevent reading Eiger hdf5 files. Hint: 'pip install hdf5plugin'")
 import fabio
 from ...utils import h5py
 import numpy as np
@@ -33,7 +29,7 @@ This can handle by default 2020-type data (all scans data in hdf5 and eiger imag
 in a per-scan hdf5 file).
 
 Example:
-    pynx-id13pty.py data=scans.h5 scan=41 detectordistance=1.3 
+    pynx-ptycho-id13 data=scans.h5 scan=41 detectordistance=1.3 
       probe=60e-6x200e-6,0.09 verbose=10 save=all saveplot liveplot
       algorithm=analysis,ML**100,DM**200,nbprobe=3,probe=1 
 
@@ -55,8 +51,6 @@ Command-line arguments (beamline-specific):
 
     nrj=8: energy in keV [mandatory, but should be in hdf5 file]
 
-    detectordistance=1.3: detector distance in meters [mandatory]
-
     pixelsize=75e-6: pixel size on detector in meters 
                      [default: 75e-6]
 
@@ -80,14 +74,20 @@ Command-line arguments (beamline-specific):
 
     kmapfile=detector/kmap/kmap_00000.edf.gz: if images are saved in a multiframe data file.
                                               This superseeds imgname=...
+
+    raster_scan=40x40: this will assume a raster scan for which regular coordinates should
+        be used. With the given number of points along each axis, the end and beginning 
+        positions will be averaged and the actual positions will be put on a regular grid.
+        NB: raster scans should not be used, so this is only provided as a workaround
+        until correct motor positions are stored in the data file.
 """
 
 # NB: for id13 we start from a flat object
 params_beamline = {'specfile': None, 'h5file': None, 'h5data': "entry_0000/measurement/data", 'nrj': None,
-                   'detectordistance': None, 'data_detector': None,
+                   'data_detector': None,
                    'pixelsize': 75e-6, 'kmapfile': None, 'monitor': None,
                    'object': 'random,0.95,1,0,0.1', 'instrument': 'ESRF id13',
-                   'ptychomotors': 'nnp2,nnp3', 'xy': '-x,y'}
+                   'ptychomotors': 'nnp2,nnp3', 'xy': '-x,y', 'raster_scan': None}
 
 params = params_generic.copy()
 for k, v in params_beamline.items():
@@ -100,7 +100,7 @@ class PtychoRunnerScanID13(PtychoRunnerScan):
 
     def load_scan(self):
         if self.params['data'] is not None:
-            # Scans are recorded in an hdf5 file
+            # Scans are recorded in a hdf5 file
             self.h = h5py.File(self.params['data'], mode='r')
             entry = self.h['%d.1' % self.scan]
             m = self.params['ptychomotors'].split(',')
@@ -116,10 +116,47 @@ class PtychoRunnerScanID13(PtychoRunnerScan):
                 self.x, self.y = eval(m[2]), eval(m[3])
             if len(self.x) < 4:
                 raise PtychoRunnerException("Less than 4 scan positions, is this a ptycho scan ?")
+            print("Read %d motor positions for %s and %s" % (len(self.x), xmot, ymot))
+            print(self.params['raster_scan'])
+            if self.params['raster_scan'] is not None:
+                nx, ny = [int(v) for v in self.params['raster_scan'].split('x')]
+                print("Interpolating motor positions for %dx%d raster scan (KLUDGE)" % (nx, ny))
+                if nx * ny != len(self.x):
+                    raise PtychoRunnerException("The number of points (%d) does not correspond to the raster "
+                                                "size: %s" % (len(self.x), params['raster_scan']))
+                # Reshape data
+                x = np.reshape(self.x, (ny, nx))
+                y = np.reshape(self.y, (ny, nx))
+                # Guess the fast axis
+                if np.std(x[0]) > np.std(y[0]):
+                    # fast axis is x
+                    # Estimate range... For the fast axis the first/last positions may be quite wrong
+                    # so need to measure the intervals and compute the extrema
+                    xc, dx = np.mean(x), np.median(x[:,1:]-x[:,:-1])
+                    x0, x1 = xc - dx * nx /2, xc + dx * nx /2
+                    # x0, x1 = np.median(x[:, 0]), np.median(x[:, -1])
+                    y0, y1 = np.median(y[0]), np.median(y[-1])
+                    print("Raster scan estimated limits: %s[x, fast axis] %6.2f->%6.2f  and %s[y] %6.2f->%6.2f "
+                          % (xmot, x0, x1, ymot, y0, y1))
+                    vx = np.linspace(x0, x1, nx)
+                    vy = np.linspace(y0, y1, ny)[:, np.newaxis]
+                else:
+                    # fast axis is y
+                    yc, dy = np.mean(y), np.median(y[:,1:]-y[:,:-1])
+                    y0, y1 = yc - dy * ny /2, yc + dy * ny /2
+                    # y0, y1 = np.median(y[:, 0]), np.median(y[:, -1])
+                    x0, x1 = np.median(x[0]), np.median(x[-1])
+                    print("Raster scan estimated limits: %s[x] %6.2f->%6.2f %s[y, fast axis] %6.2f->%6.2f "
+                          % (xmot, x0, x1, ymot, y0, y1))
+                    vx = np.linspace(x0, x1, ny)[:, np.newaxis]
+                    vy = np.linspace(y0, y1, nx)
+                self.x = (vx + 0 * vy).ravel()
+                self.y = (vx * 0 + vy).ravel()
+
             # Spec values are in microns, convert to meters
             self.x *= 1e-6
             self.y *= 1e-6
-            imgn = np.arange(len(self.x), dtype=np.int)
+            imgn = np.arange(len(self.x), dtype=int)
             if self.params['monitor'] is not None:
                 mon = entry['measurement/%s' % self.params['monitor']][()]
                 mon0 = np.median(mon)
@@ -174,7 +211,7 @@ class PtychoRunnerScanID13(PtychoRunnerScan):
             self.x *= 1e-6
             self.y *= 1e-6
 
-            imgn = np.arange(len(self.x), dtype=np.int)
+            imgn = np.arange(len(self.x), dtype=int)
 
             if self.params['monitor'] is not None:
                 if silx is not None:
@@ -242,6 +279,8 @@ class PtychoRunnerScanID13(PtychoRunnerScan):
                     # Read next hdf5 file
                     s = self.params["data_detector"] % tuple(scans + [ifile])
                     if not os.path.exists(s):
+                        if ifile == 0:
+                            raise PtychoRunnerException("Could not load data from the first detector file: %s" % s)
                         break
                     hd = h5py.File(s, mode='r')
                     hdd = hd['entry_0000/measurement/data']
@@ -354,8 +393,9 @@ class PtychoRunnerScanID13(PtychoRunnerScan):
             self.print('Time to read all frames: %4.1fs [%5.2f Mpixel/s]' % (dt, vimg.size / 1e6 / dt))
 
         # Build mask. Values of 2**32-1 and -2 are invalid (module gaps or invalidated pixels)
+        # Or is it 2**16 ??
         # TODO: mask independently the different frames
-        self.raw_mask = ((vimg > (2 ** 32 - 3)).sum(axis=0) > 0).astype(np.int8)  # very slow !
+        self.raw_mask = ((vimg > (2 ** 16 - 3)).sum(axis=0) > 0).astype(np.int8)  # very slow !
         if self.raw_mask.sum() == 0:
             self.raw_mask = None
         else:
@@ -385,10 +425,10 @@ class PtychoRunnerID13(PtychoRunner):
         """
         if k == 'h5file':
             k = 'data_detector'
-        if k in ['specfile', 'h5file', 'kmapfile', 'ptychomotors', 'data', 'data_detector']:
+        if k in ['specfile', 'h5file', 'kmapfile', 'ptychomotors', 'data', 'data_detector', 'raster_scan']:
             self.params[k] = v
             return True
-        elif k == 'nrj' or k == 'detectordistance' or k == 'pixelsize':
+        elif k in ['nrj', 'pixelsize']:
             self.params[k] = float(v)
             return True
         return False
