@@ -9,18 +9,12 @@
 from __future__ import division
 
 import sys
-import os
 import glob
 import time
 import locale
 import timeit
-from ..mpi import MPI
 
-try:
-    import hdf5plugin
-except:
-    pass
-import h5py
+from ...utils import h5py
 import numpy as np
 
 from ...utils.array import rebin
@@ -30,7 +24,7 @@ helptext_beamline = """
 Script to perform a ptychography analysis on data from nanomax@MaxIV
 
 Example:
-    pynx-nanomaxpty.py h5meta=data.h5 data=scan_%04d_pil100k_0000.hdf5 scan=9 probe=60e-6x60e-6,0.09
+    pynx-ptycho-nanomax h5meta=data.h5 data=scan_%04d_pil100k_0000.hdf5 scan=9 probe=60e-6x60e-6,0.09
       algorithm=analysis,ML**100,DM**200,nbprobe=3,probe=1
 
 Command-line arguments (beamline-specific):
@@ -66,14 +60,12 @@ Command-line arguments (beamline-specific):
                                [Mandatory]
 
 
-    detectordistance=1.3: detector distance in meters [mandatory]
-
     gpu=Titan: GPU name for calculations [Mandatory]
 """
 
 # NB: for id16 we start from a flat object (high energy, high transmission)
 params_beamline = {'h5meta': None, 'pixelsize': None, 'monitor': None, 'ptychomotors': 'samx,samy',
-                   'data': None, 'detectordistance': None, 'object': 'random,0.95,1,0,0.1',
+                   'data': None, 'object': 'random,0.95,1,0,0.1',
                    'instrument': 'NanoMAX@MaxIV', 'gpu': None, 'scan': None, 'map_rebin': None}
 
 params = params_generic.copy()
@@ -236,9 +228,9 @@ class PtychoRunnerScanNanoMAX(PtychoRunnerScan):
         if self.params['map_rebin'] is not None:
             rebinf = [int(n) for n in self.params['map_rebin'].split(',')]
             print("Rebinning scan map, rebinf: ", rebinf)
-            self.x = rebin(self.x, rebinf, normalize=True)
-            self.y = rebin(self.y, rebinf, normalize=True)
-            vimg = rebin(vimg, rebinf + [1, 1], normalize=False)
+            self.x = rebin(self.x, rebinf, scale="average")
+            self.y = rebin(self.y, rebinf, scale="average")
+            vimg = rebin(vimg, rebinf + [1, 1], scale="sum")
 
         self.x = self.x.flatten()
         self.y = self.y.flatten()
@@ -291,7 +283,7 @@ class PtychoRunnerNanoMAX(PtychoRunner):
         if k in ['h5meta', 'data', 'ptychomotors', 'map_rebin']:
             self.params[k] = v
             return True
-        elif k == 'pixelsize' or k == 'detectordistance':
+        elif k == 'pixelsize':
             self.params[k] = float(v)
             return True
         return False
