@@ -10,11 +10,7 @@ from __future__ import division
 import sys
 import timeit
 
-try:
-    import hdf5plugin
-except:
-    pass
-import h5py
+from ...utils import h5py
 import numpy as np
 
 from .runner import PtychoRunner, PtychoRunnerScan, PtychoRunnerException, params_generic
@@ -23,18 +19,19 @@ helptext_beamline = """
 Script to perform a ptychography analysis on data recorded on nanoscopium@Soleil (*.nxs format)
 
 Example:
-    pynx-nanoscopiumpty.py data=flyscan_1061-0001.nxs probe=focus,700e-6,0.63 defocus=4 detectordistance=3.265 
+    pynx-ptycho-nanoscopium data=flyscan_1061-0001.nxs probe=focus,700e-6,0.63 defocus=4 detectordistance=3.265 
                 algorithm=analysis,ML**100,AP**400,DM**100,nprobe=3,DM**200,probe=1 saveplot mask=maxipix liveplot
 
 command-line arguments:
     data=/some/dir/to/data.nxs: path to nxs data [mandatory]
-
-    scan=56: scan number 
-           [optional: this will just be used to name the output directory accordingly, otherwise 0]
+        If the data string includes one or several '%', it is assumed these are fields
+        to be replaced by the scan number, e.g. "data=flyscan_%04d-0001.nxs". This allows
+        to process series of scans using scan=6642,6643 or scan="range(13,27+1)" (note the quotes
+        which are necessary when using parenthesis in a parameter)
 
 """
 
-params_beamline = {'instrument': 'nanoscopium@Soleil', 'detectordistance': None, 'pixelsize': 55e-6, 'mask': 'maxipix',
+params_beamline = {'instrument': 'nanoscopium@Soleil', 'pixelsize': 55e-6, 'mask': 'maxipix',
                    'object': 'random,0.95,1,0,0.1', 'obj_smooth': 0.5, 'obj_inertia': 0.1, 'probe_smooth': 0.2,
                    'probe_inertia': 0.01}
 
@@ -48,7 +45,11 @@ class PtychoRunnerScanNanoscopium(PtychoRunnerScan):
         super(PtychoRunnerScanNanoscopium, self).__init__(params, scan, timings=timings)
 
     def load_scan(self):
-        h5data = h5py.File(self.params['data'], 'r')
+        filename = self.params['data']
+        if '%' in filename:
+            filename = filename % self.scan
+            self.print('Loading data:', filename)
+        h5data = h5py.File(filename, 'r')
         # Get the first entry and hope for the best
         entry = [v for v in h5data.values()][0]
 
@@ -64,7 +65,7 @@ class PtychoRunnerScanNanoscopium(PtychoRunnerScan):
         if len(self.x) < 4:
             raise PtychoRunnerException("Less than 4 scan positions, is this a ptycho scan ?")
 
-        imgn = np.arange(len(self.x), dtype=np.int)
+        imgn = np.arange(len(self.x), dtype=int)
 
         if self.params['moduloframe'] is not None:
             n1, n2 = self.params['moduloframe']
@@ -83,11 +84,15 @@ class PtychoRunnerScanNanoscopium(PtychoRunnerScan):
         self.imgn = imgn
 
     def load_data(self):
-        h5data = h5py.File(self.params['data'], 'r')
+        filename = self.params['data']
+        if '%' in filename:
+            filename = filename % self.scan
+        h5data = h5py.File(filename, 'r')
         # Get the first entry and hope for the best
         entry = [v for v in h5data.values()][0]
-        # Energy in keV must be corrected (ask Kadda !)
-        self.params['nrj'] = entry['NANOSCOPIUM/Monochromator/energy'][0] * 1.2371 - 1.3456
+        if self.params['nrj'] is None:
+            # Energy in keV must be corrected (ask Kadda !)
+            self.params['nrj'] = entry['NANOSCOPIUM/Monochromator/energy'][0] * 1.2371 - 1.3456
 
         # Load all frames
         print("Reading frames from nxs file:")
@@ -118,10 +123,11 @@ class PtychoRunnerScanNanoscopium(PtychoRunnerScan):
         self.raw_data[:, 260:, :256] = vimg[:, 256:, :256]
         self.raw_data[:, :256, 260:] = vimg[:, :256, 256:]
         self.raw_data[:, 260:, 260:] = vimg[:, 256:, 256:]
-        self.raw_data[:, 256] /= 1.5
-        self.raw_data[:, :, 256] /= 1.5
-        self.raw_data[:, 260] /= 1.5
-        self.raw_data[:, :, 260] /= 1.5
+        # TODO: check the following values - pixels should be masked anyway...
+        self.raw_data[:, 256] *= 2.5
+        self.raw_data[:, :, 256] *= 2.5
+        self.raw_data[:, 260] *= 2.5
+        self.raw_data[:, :, 260] *= 2.5
 
         print(self.x.shape, self.y.shape, self.raw_data.shape)
 
@@ -149,7 +155,7 @@ class PtychoRunnerNanoscopium(PtychoRunner):
         if k in ['data']:
             self.params[k] = v
             return True
-        elif k == 'nrj' or k == 'detectordistance':
+        elif k == 'nrj':
             self.params[k] = float(v)
             return True
         return False
@@ -163,3 +169,5 @@ class PtychoRunnerNanoscopium(PtychoRunner):
             raise PtychoRunnerException('Missing argument: no data file [data=my_data_file.nxs] given')
         if self.params['detectordistance'] is None:
             raise PtychoRunnerException('Missing argument: detectordistance=xx (in meters)')
+        if self.params['nrj'] is None:
+            raise PtychoRunnerException('Missing argument: nrj=xx (in kev)')
