@@ -10,7 +10,6 @@ from __future__ import division
 import timeit
 import numpy as np
 from ...ptycho import simulation, Ptycho, PtychoData, Calc2Obs, AP, DM, ScaleObjProbe, save_ptycho_data_cxi
-from ..shape import calc_obj_shape
 from ...mpi import MPI
 
 if MPI is not None:
@@ -21,7 +20,7 @@ helptext_beamline = """
 Script to perform a ptychography analysis on a simulated dataset
 
 Examples:
-    pynx-simulationpty.py frame_nb=128 frame_size=256 algorithm=analysis,ML**100,DM**200,nbprobe=2,probe=1 
+    pynx-ptycho-simulation frame_nb=128 frame_size=256 algorithm=analysis,ML**100,DM**200,nbprobe=2,probe=1 
                           saveplot liveplot
 
 command-line arguments:
@@ -38,7 +37,13 @@ command-line arguments:
           (x>x.mean(), y<y.mean()) will be removed. Positions are added in the other
           parts to keep the number of frames as expected. The probe used for simulation
           will also have the lower right quadrant set to zero.
-          
+    
+    simul_background=0.1: add incoherent (gaussian-shaped) background, with an integrated intensity
+        equal to the given factor multiplied by the simulated intensity. Reasonable values up to 0.2
+        [default:0 - no background]
+    
+    direct_beam: if this option is given, one frame will be with the direct beam (no sample), allowing
+        an absolute scale of the object and probe
 """
 
 # NOTE: scripts to test absolute orientation:
@@ -52,9 +57,12 @@ command-line arguments:
 # - absolute orientation of object and probe in saved plot, silx view of object, illumination and probe
 
 
-params_beamline = {'frame_nb': 128, 'frame_size': 256, 'probe': 'auto', 'siemens': False, 'logo': False, 'asym': False,
-                   'defocus': 100e-6, 'algorithm': 'ML**100,DM**200,probe=1', 'instrument': 'simulation', 'nrj': 8,
-                   'detectordistance': 1, 'pixelsize': 55e-6, 'maxsize': None, 'roi': 'full', 'autocenter': False}
+params_beamline = {'frame_nb': 128, 'frame_size': 256, 'probe': 'auto', 'siemens': False,
+                   'logo': False, 'asym': False, 'defocus': 100e-6,
+                   'algorithm': 'ML**100,AP**200,DM**200,probe=1', 'instrument': 'simulation',
+                   'nrj': 8, 'detectordistance': 1, 'pixelsize': 55e-6, 'maxsize': None,
+                   'roi': 'full', 'autocenter': False, 'saveprefix': 'none', 'simul_background': 0,
+                   'direct_beam': False}
 
 params = params_generic.copy()
 for k, v in params_beamline.items():
@@ -78,7 +86,10 @@ class PtychoRunnerScanSimul(PtychoRunnerScan):
                                   verbose=self.mpi_master)
         s.make_scan()
 
-        pixel_size_object = wavelength * detector_distance / pixel_size_detector / n
+        if self.params['near_field']:
+            pixel_size_object = pixel_size_detector
+        else:
+            pixel_size_object = wavelength * detector_distance / pixel_size_detector / n
         self.x, self.y = s.scan.values[0] * pixel_size_object, s.scan.values[1] * pixel_size_object
         self.imgn = np.arange(nb, dtype=np.int32)
         if self.params['asym']:
@@ -95,6 +106,10 @@ class PtychoRunnerScanSimul(PtychoRunnerScan):
             # Also off-center positions to check absolute position
             self.x += 2 * np.abs(self.x).max()
             self.y += 3 * np.abs(self.y).max()
+        if self.params['direct_beam']:
+            # The last frame will be with the direct beam.
+            self.x[-1] = 1e12
+            self.y[-1] = 1e12
 
     def load_data(self):
         t0 = timeit.default_timer()
@@ -109,24 +124,32 @@ class PtychoRunnerScanSimul(PtychoRunnerScan):
         else:
             obj_info = {'type': 'phase_ampl', 'phase_stretch': np.pi / 2, 'alpha_win': .2}
 
-        # The probe is calculated so that it will defocus to about 40% of the object frame size,
-        # and the step size is adapted accordingly
-        pixel_size_object = wavelength * detector_distance / (n * pixel_size_detector)
+        near_field = self.params['near_field']
+        if near_field:
+            pixel_size_object = pixel_size_detector
+        else:
+            # The probe is calculated so that it will defocus to about 40% of the object frame size,
+            # and the step size is adapted accordingly
+            pixel_size_object = wavelength * detector_distance / (n * pixel_size_detector)
         aperture = 400e-6
         focal_length = 0.1
         defocus = 0.4 * pixel_size_object * n / aperture * focal_length
-        probe_info = {'type': 'focus', 'aperture': (aperture, aperture), 'focal_length': focal_length,
-                      'defocus': defocus, 'shape': (n, n)}
-        if self.params['probe'] == 'auto':
-            # Put some randomness in the starting probe
-            r = np.random.uniform(0.9, 1.1, 4)
-            self.params['probe'] = 'focus,%ex%e,%e' % (aperture * r[0], aperture * r[1], focal_length * r[2])
-            self.params['defocus'] = defocus * r[3]
+        if near_field:
+            ap = pixel_size_detector * n * 0.7
+            probe_info = {'type': 'near_field', 'aperture': (ap, ap), 'defocus': 0.3, 'shape': (n, n)}
+        else:
+            probe_info = {'type': 'focus', 'aperture': (aperture, aperture), 'focal_length': focal_length,
+                          'defocus': defocus, 'shape': (n, n)}
+            if self.params['probe'] == 'auto':
+                # Put some randomness in the starting probe
+                r = np.random.uniform(0.9, 1.1, 4)
+                self.params['probe'] = 'focus,%ex%e,%e' % (aperture * r[0], aperture * r[1], focal_length * r[2])
+                self.params['defocus'] = defocus * r[3]
 
         # 50 scan positions correspond to 4 turns, 78 to 5 turns, 113 to 6 turns
         scan_info = {'type': 'custom', 'x': self.x / pixel_size_object, 'y': self.y / pixel_size_object}
         data_info = {'num_phot_max': 1e6, 'bg': 0, 'wavelength': wavelength, 'detector_distance': detector_distance,
-                     'detector_pixel_size': pixel_size_detector, 'noise': 'poisson'}
+                     'detector_pixel_size': pixel_size_detector, 'noise': 'poisson', 'near_field': near_field}
 
         s = simulation.Simulation(obj_info=obj_info, probe_info=probe_info, scan_info=scan_info,
                                   data_info=data_info, verbose=self.mpi_master)
@@ -143,7 +166,7 @@ class PtychoRunnerScanSimul(PtychoRunnerScan):
 
         data = PtychoData(iobs=np.ones((len(self.x), n, n), dtype=np.float32), positions=(self.x, self.y),
                           detector_distance=detector_distance, mask=None, pixel_size_detector=pixel_size_detector,
-                          wavelength=wavelength)
+                          wavelength=wavelength, near_field=near_field)
 
         if 'split' not in self.params['mpi']:
             s.make_obj()
@@ -179,6 +202,14 @@ class PtychoRunnerScanSimul(PtychoRunnerScan):
         p = Calc2Obs(nb_photons_per_frame=1e8) * p
         iobs = np.fft.fftshift(p.data.iobs, axes=(1, 2))
 
+        dark = None
+        if self.params['simul_background'] > 0:
+            x, y = p.get_probe_coord()
+            dx, dy = np.meshgrid((x - x.mean()) / (x.max() - x.min()), (y - y.mean()) / (y.max() - y.min()))
+            tmp = np.exp(-5 * (dx ** 2 + dy ** 2))
+            dark = iobs.sum() / (tmp.sum() * len(iobs)) * self.params['simul_background'] * tmp
+            iobs += dark
+
         t3 = timeit.default_timer()
         if self.timings is not None:
             dt = t3 - t2
@@ -200,6 +231,10 @@ class PtychoRunnerScanSimul(PtychoRunnerScan):
 
         # 1e8 photons on average
         self.raw_data = np.random.poisson(iobs / scale * 1e8)
+        if dark is not None:
+            dark = dark * (1e8 / scale)
+            print("Including dark with <dark>=%6.0f ph/pixel, <Iobs>=%6.0f" % (dark.mean(), self.raw_data.mean()))
+            self.dark = dark
 
         t4 = timeit.default_timer()
         if self.timings is not None:
@@ -232,12 +267,19 @@ class PtychoRunnerSimul(PtychoRunner):
         Returns:
             True if the argument is interpreted, false otherwise
         """
-        if k in ['frame_nb', 'frame_size']:
+        if k in ['frame_nb', 'frame_size', 'padding']:
             self.params[k] = int(v)
             return True
-        elif k in ['siemens', 'logo', 'asym']:
+        elif k in ['siemens', 'logo', 'asym', 'near_field']:
             self.params[k] = True
             return True
+        elif k == 'direct_beam':
+            self.params[k] = True
+            self.params['use_direct_beam'] = True
+            return True
+        elif k in ['simul_background']:
+            self.params[k] = float(v)
+            return True
         return False
 
     def check_params_beamline(self):
@@ -245,4 +287,6 @@ class PtychoRunnerSimul(PtychoRunner):
         Check if self.params includes a minimal set of valid parameters, specific to a beamiline
         Returns: Nothing. Will raise an exception if necessary
         """
+        if self.params['pixelsize'] == 55e-6 and self.params['near_field']:
+            self.params['pixelsize'] = 100e-9
         pass
