@@ -157,13 +157,15 @@ class Im(object):
             self.info.update({'type': 'Custom'})
 
     def make_even(self):
-        "Ensures even shape of the images:"
-        if len(self.values) > 2 and np.ndim(self.values) < 3:  # does not work for scan positions or for stack of images
-            psy, psx = self.values.shape
-            if psx % 2: psx -= 1
-            if psy % 2: psy -= 1
-            if (psy, psx) != self.values.shape:
-                self.values = self.values[:psx, :psy]
+        """Ensures an even shape of the images along x and y by removing one pixel when necessary"""
+        if self.values.ndim >= 2:  # Should work on 2D images as well as stack of 2D images
+            psy, psx = self.values.shape[-2:]
+            if psx % 2:
+                psx -= 1
+            if psy % 2:
+                psy -= 1
+            if (psy, psx) != self.values.shape[-2:]:
+                self.values = self.values[..., :psy, :psx]
                 self.info['shape'] = self.values.shape
                 self.info['type'] += 'Even'
                 print("Changing shape to even values: (%g,%g)" % self.values.shape)
@@ -308,7 +310,7 @@ class Simulation(object):
         default_scan_info = {'type': 'spiral',
                              'scan_step_pix': 30,
                              'n_scans': 50,
-                             'integer_values': True}
+                             'integer_values': False}
         default_scan_info.update(self.scan.info)
         self.scan.info = default_scan_info.copy()
 
@@ -351,6 +353,10 @@ class Simulation(object):
         self.probe.make_even()
 
         posx, posy = self.scan.values
+        # Handle values>1e10 indicating a frame with the direct beam
+        pos_mask = np.logical_or(abs(posx) > 1e10, abs(posy) > 1e10)
+        posx = np.ma.masked_array(posx, pos_mask, fill_value=1e10)
+        posy = np.ma.masked_array(posy, pos_mask, fill_value=1e10)
         posy_max, posx_max = np.ceil(abs(posy).max()), np.ceil(abs(posx).max())
         n = len(posx)  # number of scan positions
 
@@ -396,16 +402,21 @@ class Simulation(object):
 
         nb_photons_per_frame = self.data_info['nb_photons_per_frame']
 
+        # Mask beamstop data with <0 intensities
+        tmp = np.isclose(beam_stop, 0).astype(int)
+        if tmp.sum():
+            intensity = intensity * (1 - tmp) - 100 * tmp
+
         # Use Ptycho code for simulation
         pd = ptycho.PtychoData(iobs=intensity, positions=(posx * pixel_size_object, posy * pixel_size_object),
                                detector_distance=detector_distance, wavelength=wavelength, near_field=near_field,
-                               mask=np.isclose(beam_stop, 0), pixel_size_detector=pixel_size_detector)
+                               pixel_size_detector=pixel_size_detector)
         p = ptycho.Ptycho(probe=self.probe.values, obj=self.obj.values, data=pd)
         p = ptycho.Calc2Obs(nb_photons_per_frame=nb_photons_per_frame,
                             poisson_noise=(self.data_info['noise'] == 'poisson')) * p
         intensity = np.fft.fftshift(p.data.iobs, axes=(1, 2)) * beam_stop
         if rf > 1:  # rebin_factor
-            intensity = rebin(intensity, (1, rf, rf), normalize=False)
+            intensity = rebin(intensity, (1, rf, rf), scale="sum")
             if self.verbose:
                 print("\nBinning data by %s" % rf)
 
@@ -447,13 +458,38 @@ class Simulation(object):
             else:
                 # Tile the object to fill the area - deactivated as the original image is 512 pixels, so
                 # a size likely similar to the probe, so may produce weird periodic artefacts
-                ny, nx = obj_shape
                 ny0, nx0 = self.obj.values.shape
                 tmp = np.tile(self.obj.values, (int(np.ceil(ny / ny0)), int(np.ceil(nx / nx0))))
                 ny1, nx1 = tmp.shape
                 tmp = tmp[ny1 // 2 - ny // 2:ny1 // 2 - ny // 2 + ny,
                       nx1 // 2 - nx // 2:nx1 // 2 - nx // 2 + nx]
 
+        if self.obj.info['type'] == 'transmission':
+            # First transform this into a spherical object with
+            # some variation in the density
+            ny, nx = tmp.shape
+            x = np.linspace(-nx // 2, nx // 2, nx)
+            y = np.linspace(-ny // 2, ny // 2, ny)
+            xx, yy = np.meshgrid(x, y)
+            r = np.sqrt(xx ** 2 + yy ** 2)
+            t = np.sqrt(np.fmax(1 - r / (0.3 * nx), 0))
+            tmp = t * (1 - 0.2 * tmp / tmp.max())
+
+            # Now compute this as a transmission objct
+            delta_beta = self.obj.info['delta_beta']
+            wavelength = self.obj.info['wavelength']
+            beta = 1e-9
+            delta = delta_beta * beta
+            # mu = 4 * np.pi * beta / wavelength
+            k = 2 * np.pi / wavelength
+            if 'phase_stretch' in self.obj.info:
+                dph = self.obj.info['phase_stretch']
+            else:
+                dph = np.pi
+            # Required thickness to reach desired delta_phi
+            thickness = dph / abs(k * delta * tmp.real).max()
+            tmp = np.exp(k * (-1j * delta - beta) * thickness * tmp.real)
+
         if 'alpha_win' in self.obj.info:
             s = tmp.shape
             mi = abs(tmp).min()
@@ -472,6 +508,12 @@ class Simulation(object):
         if obj_type == 'custom':
             obj = self.obj.values
 
+        elif obj_type == 'transmission':
+            # Just set the thickness of the object here, real transmission is computed in make_obj_true()
+            assert 'delta_beta' in self.obj.info, "'delta_beta' must be supplied for transmission objects"
+            assert 'wavelength' in self.obj.info, "'wavelength' must be supplied for transmission objects"
+            obj = self.DTYPE_REAL(get_img(0))
+
         elif ('ampl' in obj_type) and ('phase' in obj_type):
             im0 = np.flipud(self.DTYPE_REAL(get_img(0)))
             im1 = np.flipud(self.DTYPE_REAL(get_img(1)))
@@ -616,31 +658,22 @@ class Simulation(object):
             w = wavefront.FreePU() * w
 
         elif probe_type == 'near_field':
-            defocus = self.probe.info['defocus']
             nx, ny = self.probe.info['shape']
             assert nx == ny
             # Pixel size at focus
             pixel_size_detector = self.data_info['detector_pixel_size']
+            amp = 10
+            if 'amplitude' in info:
+                amp = info['amplitude']
 
             assert self.wavelength is not None, "Wavelength is None !?"
-
-            # Pixel size at aperture
-            d = np.zeros((nx, nx), dtype=np.complex64)
-            self.w = wavefront.Wavefront(d=d, wavelength=self.wavelength, pixel_size=pixel_size_detector)
-            x, y = self.w.get_x_y()
-            if len(info['aperture']) == 1:
-                # Circular aperture
-                radius = info['aperture'][0]
-                r = np.sqrt(x ** 2 + y ** 2)
-                self.w.set(r < radius)
-            else:
-                # Rectangular slit
-                widthx, widthy = info['aperture']
-                self.w.set((abs(y) < (widthy / 2)) * (abs(x) < (widthx / 2)))
-            w = wavefront.PropagateNearField(dz=defocus) * self.w
-            self.probe.values = self.w.get(shift=True)
-            w = wavefront.FreePU() * w
-
+            # Use holotomo function
+            from ..holotomo.utils import simulate_probe
+            # propagate to the maximum distance
+            dz = (pixel_size_detector * nx) ** 2 / self.wavelength / nx
+            self.probe.values = simulate_probe((ny, nx), dz=[dz], pixel_size=pixel_size_detector,
+                                               wavelength=self.wavelength, nb_line_v=10, nb_line_h=10,
+                                               nb_spot=10, defect_amplitude=amp, amplitude=amp).squeeze()
         else:
             msg = "Unknown probe type:", self.probe.info['type']
             raise NameError(msg)
@@ -651,15 +684,13 @@ class Simulation(object):
         if self.verbose:
             print("Simulating scan: %s" % info['type'])
         scan_type = info['type'].lower()
-        if scan_type == 'rect':
-            scanX = np.linspace(0, self.obj.values.shape[0], info['scan_step_pix'])
-            scanY = np.linspace(0, self.obj.values.shape[1], info['scan_step_pix'])
-            if scanX.size == 0:
-                scanX = np.array([0])  # to make at least one scan
-            if scanY.size == 0:
-                scanY = np.array([0])
-            posx = scanX - scanX.mean()
-            posy = scanY - scanY.mean()
+        if scan_type in ['rect', 'raster']:
+            posy = np.arange(0, self.obj.values.shape[-2], info['scan_step_pix'])
+            posx = np.arange(0, self.obj.values.shape[-1], info['scan_step_pix'])
+            posy, posx = np.meshgrid(posy, posx, indexing='ij')
+            posy, posx = posy.ravel(), posx.ravel()
+            posx -= posx.mean()
+            posy -= posy.mean()
 
         elif scan_type == 'spiral':
             posy, posx = spiral_archimedes(info['scan_step_pix'], info['n_scans'])
