@@ -49,7 +49,7 @@ def make_ptycho_data(dsize=256, nb_frame=100, nb_photons=1e9):
     wavelength = 1.5e-10
     detector_distance = 1
     obj_info = {'type': 'phase_ampl', 'phase_stretch': np.pi / 2, 'alpha_win': .2}
-    probe_info = {'type': 'focus', 'aperture': (60e-6, 60e-6), 'focal_length': .08, 'defocus': 400e-6,
+    probe_info = {'type': 'focus', 'aperture': (200e-6, 200e-6), 'focal_length': .08, 'defocus': 450e-6,
                   'shape': (dsize, dsize)}
 
     # 50 scan positions correspond to 4 turns, 78 to 5 turns, 113 to 6 turns
@@ -76,6 +76,11 @@ def make_ptycho_data_cxi(dsize=256, nb_frame=100, nb_photons=1e9, dir_name=None)
     f, path = tempfile.mkstemp(suffix='.cxi', prefix="TestPtycho", dir=dir_name)
     save_ptycho_data_cxi(path, iobs, pixel_size_detector, wavelength, detector_distance, x, y, z=None,
                          monitor=None, mask=None, instrument="Simulation", overwrite=True)
+    # Also make a mask and flatfield
+    mask = np.random.uniform(0, 1, (dsize, dsize)) < 0.05
+    flatfield = np.random.uniform(0.98, 1.02, (dsize, dsize))
+    np.savez_compressed(os.path.join(dir_name, 'mask.npz'), mask=mask)
+    np.savez_compressed(os.path.join(dir_name, 'flatfield.npz'), flatfield=flatfield)
     return path
 
 
@@ -83,17 +88,25 @@ class TestPtycho(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        cls.tmp_dir = tempfile.mkdtemp()
+        # Directory contents will automatically get cleaned up on deletion
+        cls.tmp_dir_obj = tempfile.TemporaryDirectory()
+        cls.tmp_dir = cls.tmp_dir_obj.name
         # Ptycho data and main object, created in make_ptycho_data and make_ptycho_obj
         cls.data = None
         cls.p = None
         cls.obj0 = None
+        cls.illum_mask = None  # Object illumination
         cls.probe0 = None
+        cls.ops = []
+        if 'cuda' not in sys.argv and 'opencl' not in sys.argv:
+            cls.ops.append(cpuop)
+        if has_cuda and 'opencl' not in sys.argv:
+            cls.ops.append(cuop)
+        if has_opencl and 'cuda' not in sys.argv:
+            cls.ops.append(clop)
 
     @classmethod
     def tearDownClass(cls):
-        # print("Removing temporary directory: %s" % (cls.tmp_dir))
-        shutil.rmtree(cls.tmp_dir)
         if cls.data is not None:
             del cls.data
         if cls.p is not None:
@@ -123,7 +136,7 @@ class TestPtycho(unittest.TestCase):
             nyo, nxo = shape.calc_obj_shape(self.data.posx / px, self.data.posy / px, self.data.iobs.shape[1:])
 
             obj_init_info = {'type': 'random', 'range': (0, 1, 0, 0.5), 'shape': (nyo, nxo)}
-            probe_init_info = {'type': 'focus', 'aperture': (70e-6, 70e-6), 'focal_length': .08, 'defocus': 400e-6,
+            probe_init_info = {'type': 'focus', 'aperture': (230e-6, 230e-6), 'focal_length': .08, 'defocus': 460e-6,
                                'shape': (n, n)}
             data_info = {'wavelength': self.data.wavelength, 'detector_distance': self.data.detector_distance,
                          'detector_pixel_size': pxd}
@@ -140,6 +153,8 @@ class TestPtycho(unittest.TestCase):
             # Keep these object and probe as a reference
             self.obj0 = self.p.get_obj()
             self.probe0 = self.p.get_probe()
+            illum = self.p.get_illumination_obj()
+            self.illum_mask = illum > (illum.max() * 0.1)
 
     def test_00_make_ptycho_data(self):
         self.make_ptycho_data()
@@ -171,7 +186,9 @@ class TestPtycho(unittest.TestCase):
         # Compute LLK before and after
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = clop.AP(update_object=True, update_probe=False, calc_llk=0) ** 10 * self.p
         self.p = clop.AP(update_object=True, update_probe=True, calc_llk=0) ** 10 * self.p
+        self.p = clop.AP(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
         # print("AP**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
@@ -186,7 +203,9 @@ class TestPtycho(unittest.TestCase):
         # Compute LLK before and after
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = clop.DM(update_object=True, update_probe=False, calc_llk=0) ** 10 * self.p
         self.p = clop.DM(update_object=True, update_probe=True, calc_llk=0) ** 10 * self.p
+        self.p = clop.DM(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
         # print("DM**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
@@ -201,7 +220,27 @@ class TestPtycho(unittest.TestCase):
         # Compute LLK before and after
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = clop.ML(update_object=True, update_probe=False, calc_llk=0) ** 10 * self.p
         self.p = clop.ML(update_object=True, update_probe=True, calc_llk=0) ** 10 * self.p
+        self.p = clop.ML(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
+        self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
+        llk1 = self.p.llk_poisson / self.p.nb_obs
+        # print("ML**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
+        self.p = clop.FreePU() * self.p
+
+    @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
+    def test_ML_regul_opencl(self):
+        self.make_ptycho_obj()
+        self.p.set_obj(self.obj0)
+        self.p.set_probe(self.probe0)
+        # Compute LLK before and after
+        self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
+        llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = clop.ML(update_object=True, update_probe=True, calc_llk=0, reg_fac_probe=0.1) ** 10 * self.p
+        self.p = clop.ML(update_object=True, update_probe=True, calc_llk=0, reg_fac_obj=0.1) ** 10 * self.p
+        self.p = clop.ML(update_object=True, update_probe=True, calc_llk=0,
+                         reg_fac_obj=0.1, reg_fac_probe=0.1) ** 10 * self.p
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
         # print("ML**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
@@ -218,7 +257,7 @@ class TestPtycho(unittest.TestCase):
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
         self.p = clop.AP(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
-        # self.p = clop.ML(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
+        self.p = clop.ML(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
         self.p = clop.LoopStack(clop.PropagateApplyAmplitude(calc_llk=True) * clop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
         # print("ML**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
@@ -264,18 +303,19 @@ class TestPtycho(unittest.TestCase):
         self.p = clop.ObjProbe2Psi() * self.p
         psi0 = self.p._cl_psi.get()
         self.p = clop.FT(scale=False) * self.p
-        psi1 = self.p._cl_psi.get()
+        s = clop.default_processing_unit.fft_scale(self.p._cl_psi.shape, ndim=2)
+        psi1 = self.p._cl_psi.get() * s[0]
         self.p = clop.IFT(scale=False) * self.p
-        psi2 = self.p._cl_psi.get()
+        psi2 = self.p._cl_psi.get() * s[0] * s[1]
         self.p = clop.FreePU() * self.p
         # Check that L2 norms have the expected behaviour
         npsi0 = (np.abs(psi0) ** 2).sum()
         npsi1 = (np.abs(psi1) ** 2).sum()
         npsi2 = (np.abs(psi2) ** 2).sum()
-        message = "OpenCL (clFFT/gpyfft) FT scaling should be sqrt(n)"
-        self.assertAlmostEqual(npsi1 / npsi0, psi0[0, 0, 0].size, msg=message, delta=1e-5 * psi0[0, 0, 0].size)
-        message = "OpenCL (clFFT/gpyfft) FT-1 scaling should be 1/sqrt(n)"
-        self.assertAlmostEqual(npsi1 / npsi2, psi0[0, 0, 0].size, msg=message, delta=1e-5 * psi0[0, 0, 0].size)
+        message = "Checking FT scale: %7.3f %7.3f" % (npsi1 / npsi0, npsi2 / npsi1)
+        self.assertAlmostEqual(npsi1 / npsi0, 1, msg=message, delta=1e-5)
+        message = "Checking IFT scale: %7.3f %7.3f" % (npsi1 / npsi0, npsi2 / npsi1)
+        self.assertAlmostEqual(npsi1 / npsi2, 1, msg=message, delta=1e-5)
 
     @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
     @unittest.skipIf(has_opencl is False, 'no OpenCL support')
@@ -297,6 +337,7 @@ class TestPtycho(unittest.TestCase):
         # Compute LLK before and after
         self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = cuop.AP(update_object=True, update_probe=False, calc_llk=0) ** 10 * self.p
         self.p = cuop.AP(update_object=True, update_probe=True, calc_llk=0) ** 10 * self.p
         self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
@@ -312,7 +353,9 @@ class TestPtycho(unittest.TestCase):
         # Compute LLK before and after
         self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = cuop.DM(update_object=True, update_probe=False, calc_llk=0) ** 10 * self.p
         self.p = cuop.DM(update_object=True, update_probe=True, calc_llk=0) ** 10 * self.p
+        self.p = cuop.DM(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
         self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
         # print("DM**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
@@ -333,6 +376,24 @@ class TestPtycho(unittest.TestCase):
         # print("ML**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
         self.p = cuop.FreePU() * self.p
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_ML_regul_cuda(self):
+        self.make_ptycho_obj()
+        self.p.set_obj(self.obj0)
+        self.p.set_probe(self.probe0)
+        # Compute LLK before and after
+        self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
+        llk0 = self.p.llk_poisson / self.p.nb_obs
+        self.p = cuop.ML(update_object=True, update_probe=True, calc_llk=0, reg_fac_probe=0.1) ** 10 * self.p
+        self.p = cuop.ML(update_object=True, update_probe=True, calc_llk=0, reg_fac_obj=0.1) ** 10 * self.p
+        self.p = cuop.ML(update_object=True, update_probe=True, calc_llk=0,
+                         reg_fac_obj=0.1, reg_fac_probe=0.1) ** 10 * self.p
+        self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
+        llk1 = self.p.llk_poisson / self.p.nb_obs
+        # print("ML**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
+        self.p = cuop.FreePU() * self.p
+
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
     def test_background_AP_ML_cuda(self):
@@ -344,7 +405,7 @@ class TestPtycho(unittest.TestCase):
         self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
         llk0 = self.p.llk_poisson / self.p.nb_obs
         self.p = cuop.AP(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
-        # self.p = cuop.ML(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
+        self.p = cuop.ML(update_object=True, update_probe=True, update_background=True, calc_llk=0) ** 10 * self.p
         self.p = cuop.LoopStack(cuop.PropagateApplyAmplitude(calc_llk=True) * cuop.ObjProbe2Psi()) * self.p
         llk1 = self.p.llk_poisson / self.p.nb_obs
         # print("ML**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
@@ -390,18 +451,19 @@ class TestPtycho(unittest.TestCase):
         self.p = cuop.ObjProbe2Psi() * self.p
         psi0 = self.p._cu_psi.get()
         self.p = cuop.FT(scale=False) * self.p
-        psi1 = self.p._cu_psi.get()
+        s = cuop.default_processing_unit.fft_scale(self.p._cu_psi.shape, ndim=2)
+        psi1 = self.p._cu_psi.get() * s[0]
         self.p = cuop.IFT(scale=False) * self.p
-        psi2 = self.p._cu_psi.get()
+        psi2 = self.p._cu_psi.get() * s[0] * s[1]
         self.p = cuop.FreePU() * self.p
         # Check that L2 norms have the expected behaviour
         npsi0 = (np.abs(psi0) ** 2).sum()
         npsi1 = (np.abs(psi1) ** 2).sum()
         npsi2 = (np.abs(psi2) ** 2).sum()
-        message = "CUDA FT scaling should be sqrt(n)"
-        self.assertAlmostEqual(npsi1 / npsi0, psi0[0, 0, 0].size, msg=message, delta=1e-5 * psi0[0, 0, 0].size)
-        message = "CUDA FT-1 scaling should be sqrt(n)"
-        self.assertAlmostEqual(npsi2 / npsi1, psi0[0, 0, 0].size, msg=message, delta=1e-5 * psi0[0, 0, 0].size)
+        message = "Checking FT scale: %7.3f %7.3f" % (npsi1 / npsi0, npsi2 / npsi1)
+        self.assertAlmostEqual(npsi1 / npsi0, 1, msg=message, delta=1e-5)
+        message = "Checking IFT scale: %7.3f %7.3f" % (npsi1 / npsi0, npsi2 / npsi1)
+        self.assertAlmostEqual(npsi2 / npsi1, 1, msg=message, delta=1e-5)
 
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
@@ -433,6 +495,25 @@ class TestPtycho(unittest.TestCase):
         self.assertTrue(np.allclose(psicl, psicu, rtol=1e-4, atol=atol),
                         msg="CUDA and OpenCL Psi2ObjProbe must give close results")
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_consistency_cuda_cpu_ObjProbe2Psi(self):
+        i = 0
+        self.make_ptycho_obj()
+        for near_field in [False, True]:
+            with self.subTest(near_field=near_field):
+                self.p.set_obj(self.obj0)
+                self.p.set_probe(self.probe0)
+                self.p = cpuop.ObjProbe2Psi() * self.p
+                psicpu = self.p._psi[0, 0, 0]
+                self.p = cuop.ObjProbe2Psi() * cuop.SelectStack(i) * self.p
+                psicu = self.p._cu_psi.get()[0, 0, 0]
+                atol = np.abs(psicpu).max() * 1e-4
+                self.p = cuop.FreePU() * self.p
+                self.assertTrue(np.allclose(psicpu, psicu, rtol=1e-4, atol=atol),
+                                msg="CUDA and CPU Psi2ObjProbe must give close results")
+        self.p.data.near_field = False
+
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
     @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
@@ -452,6 +533,27 @@ class TestPtycho(unittest.TestCase):
         self.assertTrue(np.allclose(psicl, psicu, rtol=1e-4, atol=atol),
                         msg="CUDA and OpenCL FT*Psi2ObjProbe must give close results")
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_consistency_cuda_cpu_FT_ObjProbe2Psi(self):
+        i = 0
+        self.make_ptycho_obj()
+        for near_field in [False, True]:
+            with self.subTest(near_field=near_field):
+                # KLUDGE: is it safe to change near_field that way ?
+                self.p.data.near_field = near_field
+                self.p.set_obj(self.obj0)
+                self.p.set_probe(self.probe0)
+                self.p = cpuop.FT(scale=True) * cpuop.ObjProbe2Psi() * self.p
+                psicpu = self.p._psi[0, 0, 0]
+                self.p = cuop.FT(scale=True) * cuop.ObjProbe2Psi() * cuop.SelectStack(i) * self.p
+                psicu = self.p._cu_psi.get()[0, 0, 0]
+                atol = np.abs(psicpu).max() * 1e-4
+                self.p = cuop.FreePU() * self.p
+                self.assertTrue(np.allclose(psicpu, psicu, rtol=1e-4, atol=atol),
+                                msg="CUDA and CPU FT*Psi2ObjProbe must give close results")
+        self.p.data.near_field = False
+
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
     @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
@@ -471,12 +573,50 @@ class TestPtycho(unittest.TestCase):
         self.assertTrue(np.allclose(psicl, psicu, rtol=1e-4, atol=atol),
                         msg="CUDA and OpenCL PropagateApplyAmplitude must give close results")
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_consistency_cuda_cpu_ApplyAmplitude(self):
+        i = 0
+        self.make_ptycho_obj()
+        for near_field in [False, True]:
+            with self.subTest(near_field=near_field):
+                self.p.set_obj(self.obj0)
+                self.p.set_probe(self.probe0)
+                self.p = cpuop.ApplyAmplitude() * cpuop.FT(scale=True) * cpuop.ObjProbe2Psi() * self.p
+                psicpu = self.p._psi[0, 0, 0]
+                self.p = cuop.ApplyAmplitude() * cuop.FT(scale=True) * \
+                         cuop.ObjProbe2Psi() * cuop.SelectStack(i) * self.p
+                psicu = self.p._cu_psi.get()[0, 0, 0]
+                atol = np.abs(psicpu).max() * 1e-3
+                self.p = cuop.FreePU() * self.p
+                self.assertTrue(np.allclose(psicpu, psicu, rtol=1e-3, atol=atol),
+                                msg="CUDA and CPU PropagateApplyAmplitude must give close results")
+        self.p.data.near_field = False
+
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_consistency_cuda_cpu_PropagateApplyAmplitude(self):
+        i = 0
+        self.make_ptycho_obj()
+        for near_field in [False, True]:
+            with self.subTest(near_field=near_field):
+                self.p.set_obj(self.obj0)
+                self.p.set_probe(self.probe0)
+                self.p = cpuop.PropagateApplyAmplitude() * cpuop.ObjProbe2Psi() * self.p
+                psicpu = self.p._psi[0, 0, 0]
+                self.p = cuop.PropagateApplyAmplitude() * cuop.ObjProbe2Psi() * cuop.SelectStack(i) * self.p
+                psicu = self.p._cu_psi.get()[0, 0, 0]
+                atol = np.abs(psicpu).max() * 1e-3
+                self.p = cuop.FreePU() * self.p
+                self.assertTrue(np.allclose(psicpu, psicu, rtol=1e-3, atol=atol),
+                                msg="CUDA and CPU PropagateApplyAmplitude must give close results")
+        self.p.data.near_field = False
+
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
     @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
     @unittest.skipIf(has_opencl is False, 'no OpenCL support')
-    def test_consistency_opencl_cuda_AP(self):
-        i = 0
+    def test_consistency_opencl_cuda_AP(self, rtol=1e-4):
         self.make_ptycho_obj()
         self.p.set_obj(self.obj0)
         self.p.set_probe(self.probe0)
@@ -488,20 +628,43 @@ class TestPtycho(unittest.TestCase):
         self.p = cuop.AP(update_probe=True, update_object=True) ** 5 * self.p
         objcu = self.p.get_obj()
         probecu = self.p.get_probe()
-        atolo = np.abs(objcl).max() * 1e-4
-        atolp = np.abs(probecl).max() * 1e-4
+        atolo = np.abs(objcl).max() * rtol
+        atolp = np.abs(probecl).max() * rtol
         self.p = clop.FreePU() * self.p
         self.p = cuop.FreePU() * self.p
-        self.assertTrue(np.allclose(objcl, objcu, rtol=1e-4, atol=atolo)
-                        and np.allclose(probecl, probecu, rtol=1e-4, atol=atolp),
+        self.assertTrue(np.allclose(objcl, objcu, rtol=rtol, atol=atolo)
+                        and np.allclose(probecl, probecu, rtol=rtol, atol=atolp),
                         msg="CUDA and OpenCL AP**5 must give close results")
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_consistency_cuda_cpu_AP(self, rtol=1e-3):
+        self.make_ptycho_obj()
+        for near_field in [False, True]:
+            with self.subTest(near_field=near_field):
+                self.p.set_obj(self.obj0.copy())
+                self.p.set_probe(self.probe0.copy())
+                self.p = cpuop.AP(update_probe=True, update_object=True) ** 5 * self.p
+                objcpu = self.p.get_obj().copy()
+                probecpu = self.p.get_probe().copy()
+                self.p.set_obj(self.obj0.copy())
+                self.p.set_probe(self.probe0.copy())
+                self.p = cuop.AP(update_probe=True, update_object=True) ** 5 * self.p
+                objcu = self.p.get_obj()
+                probecu = self.p.get_probe()
+                atolo = np.abs(objcpu).max() * rtol
+                atolp = np.abs(probecpu).max() * rtol
+                self.p = cuop.FreePU() * self.p
+                self.assertTrue(np.allclose(objcpu, objcu, rtol=rtol, atol=atolo),
+                                msg="CUDA and CPU AP must give close objects")
+                self.assertTrue(np.allclose(probecpu, probecu, rtol=rtol, atol=atolp),
+                                msg="CUDA and CPU AP must give close probes")
+
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
     @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
     @unittest.skipIf(has_opencl is False, 'no OpenCL support')
-    def test_consistency_opencl_cuda_AP(self):
-        i = 0
+    def test_consistency_opencl_cuda_DM(self, rtol=1e-3):
         self.make_ptycho_obj()
         self.p.set_obj(self.obj0)
         self.p.set_probe(self.probe0)
@@ -513,14 +676,36 @@ class TestPtycho(unittest.TestCase):
         self.p = cuop.DM(update_probe=True, update_object=True) ** 5 * self.p
         objcu = self.p.get_obj()
         probecu = self.p.get_probe()
-        atolo = np.abs(objcl).max() * 1e-4
-        atolp = np.abs(probecl).max() * 1e-4
+        atolo = np.abs(objcl).max() * rtol
+        atolp = np.abs(probecl).max() * rtol
         self.p = clop.FreePU() * self.p
         self.p = cuop.FreePU() * self.p
-        self.assertTrue(np.allclose(objcl, objcu, rtol=1e-4, atol=atolo)
-                        and np.allclose(probecl, probecu, rtol=1e-4, atol=atolp),
+        self.assertTrue(np.allclose(objcl, objcu, rtol=rtol, atol=atolo)
+                        and np.allclose(probecl, probecu, rtol=rtol, atol=atolp),
                         msg="CUDA and OpenCL DM**5 must give close results")
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_consistency_cuda_cpu_DM(self, rtol=1e-3):
+        self.make_ptycho_obj()
+        self.p.set_obj(self.obj0.copy())
+        self.p.set_probe(self.probe0.copy())
+        self.p = cpuop.DM(update_probe=True, update_object=True, zero_phase_ramp=False) ** 5 * self.p
+        objcpu = self.p.get_obj().copy()[0] * self.illum_mask
+        probecpu = self.p.get_probe().copy()
+        self.p.set_obj(self.obj0.copy())
+        self.p.set_probe(self.probe0.copy())
+        self.p = cuop.DM(update_probe=True, update_object=True, zero_phase_ramp=False) ** 5 * self.p
+        objcu = self.p.get_obj()[0] * self.illum_mask
+        probecu = self.p.get_probe()
+        atolo = np.abs(objcpu).max() * rtol
+        atolp = np.abs(probecpu).max() * rtol
+        self.p = cuop.FreePU() * self.p
+        self.assertTrue(np.allclose(objcpu, objcu, rtol=rtol, atol=atolo),
+                        msg="CUDA and CPU DM**5 must give close objects")
+        self.assertTrue(np.allclose(probecpu, probecu, rtol=rtol, atol=atolp),
+                        msg="CUDA and CPU DM**5 must give close probes")
+
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
     @unittest.skipIf('cuda' in sys.argv or exclude_cuda, "OpenCL tests excluded")
@@ -531,15 +716,22 @@ class TestPtycho(unittest.TestCase):
         self.p.set_probe(self.probe0)
         self.p._cl_obj_grad = clop.cla.empty(clop.default_processing_unit.cl_queue, self.p._obj.shape, np.complex64)
         self.p._cl_probe_grad = clop.cla.empty(clop.default_processing_unit.cl_queue, self.p._probe.shape, np.complex64)
+        self.p._cl_background_grad = clop.cla.zeros(clop.default_processing_unit.cl_queue, self.p._background.shape,
+                                                    np.float32)
+        self.p._cl_background_dir = clop.cla.zeros(clop.default_processing_unit.cl_queue, self.p._background.shape,
+                                                   np.float32)
         self.p = clop.Grad(update_probe=True, update_object=True) * self.p
-        objcl = self.p._cl_obj_grad.get()
+        illum = self.p.get_illumination_obj()
+        objcl = self.p._cl_obj_grad.get() * illum
         probecl = self.p._cl_probe_grad.get()
         self.p.set_obj(self.obj0)
         self.p.set_probe(self.probe0)
         self.p._cu_obj_grad = cuop.cua.empty(self.p._obj.shape, dtype=np.complex64)
         self.p._cu_probe_grad = cuop.cua.empty(self.p._probe.shape, dtype=np.complex64)
+        self.p._cu_background_grad = cua.zeros(self.p._background.shape, dtype=np.float32)
+        self.p._cu_background_dir = cua.zeros(self.p._background.shape, dtype=np.float32)
         self.p = cuop.Grad(update_probe=True, update_object=True) * self.p
-        objcu = self.p._cu_obj_grad.get()
+        objcu = self.p._cu_obj_grad.get() * illum
         probecu = self.p._cu_probe_grad.get()
         atolo = np.abs(objcl).max() * rtol
         atolp = np.abs(probecl).max() * rtol
@@ -645,6 +837,18 @@ class TestPtycho(unittest.TestCase):
         self.assertTrue(np.allclose(cu_n, cpu_n, rtol=rtol, atol=atol),
                         msg="CUDA and CPU CalcIllumination must give close results")
 
+    def test_get_iobs_icalc(self):
+        self.make_ptycho_obj()
+        self.p.set_obj(self.obj0)
+        self.p.set_probe(self.probe0)
+        for op in self.ops:
+            with self.subTest(op=op):
+                for i in [0, len(self.p.data.iobs) - 1]:
+                    iobs = self.p.get_iobs(i)
+                    iobs = self.p.get_iobs(i, shift=True)
+                    icalc = op.get_icalc(self.p, i)
+                    icalc = op.get_icalc(self.p, i, shift=True)
+
 
 def suite():
     test_suite = unittest.TestSuite()
