@@ -39,9 +39,10 @@ def fhkl(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False,
     s = "PASS"
     if tmp > 0.01 or isnan(tmp):
         s = "FAIL"
-    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
-          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
-          ("fhkl", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    if verbose:
+        print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+              "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
+              ("fhkl", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     return tmp < 0.01
 
 
@@ -69,9 +70,10 @@ def fhklo(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, lan
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
     if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
-          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % (
-              "fhklo", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    if verbose:
+        print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+              "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % (
+                  "fhklo", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     if return_array:
         return diff
     return tmp < 0.01
@@ -102,10 +104,11 @@ def fhklo_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
     if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
-          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
-          ("fhklo_graz", nh * nk * nl // 1000, nx * ny * nz // 1000,
-           nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    if verbose:
+        print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+              "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
+              ("fhklo_graz", nh * nk * nl // 1000, nx * ny * nz // 1000,
+               nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     if return_array:
         return diff
     return tmp < 0.01
@@ -135,10 +138,11 @@ def fhkl_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False,
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
     if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
-          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
-          ("fhkl_graz", nh * nk * nl // 1000, nx * ny * nz // 1000,
-           nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    if verbose:
+        print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+              "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
+              ("fhkl_graz", nh * nk * nl // 1000, nx * ny * nz // 1000,
+               nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     if return_array:
         return diff
     return tmp < 0.01
@@ -186,8 +190,9 @@ def dwba4(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform
     # Ux and Uy displacements (relative to Si)
     ux = x * (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50))
     uy = y * (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50))
-    if verbose: print("Simulation with epsilon_xx=epsilon_yy going from %4.2f%% to %4.2f%%" % (
-        0.5, (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50)).max() * 100))
+    if verbose:
+        print("Simulation with epsilon_xx=epsilon_yy going from %4.2f%% to %4.2f%%" % (
+            0.5, (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50)).max() * 100))
 
     # Compute refraction index for substrate
     si = gid.Scatterer("Si", (0., 0., 0.), 1.0, 1.0 / (8 * pi ** 2), nrj)
@@ -232,9 +237,10 @@ def dwba4(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform
     s = "PASS"
     tmp = abs(fhkl_).mean()
     if tmp < 1 or isnan(tmp): s = "FAIL"
-    print("%20s: simple DWBA test, 4 paths, using cctbx for refraction index calculations "
-          "(no strict check)                                         %10s" %
-          ("pynx.gid.FhklDWBA4", s))
+    if verbose:
+        print("%20s: simple DWBA test, 4 paths, using cctbx for refraction index calculations "
+              "(no strict check)                                         %10s" %
+              ("pynx.gid.FhklDWBA4", s))
     if return_array:
         return fhkl
     return abs(fhkl_).mean() > 1
@@ -291,9 +297,10 @@ def dwba5(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform
     s = "PASS"
     tmp = abs(fhkl_).mean()
     if tmp < 1 or isnan(tmp): s = "FAIL"
-    print("%20s: simple DWBA test, 5 paths, using cctbx for refraction index calculations "
-          "(no strict check)                                         %10s" %
-          ("pynx.gid.FhklDWBA5", s))
+    if verbose:
+        print("%20s: simple DWBA test, 5 paths, using cctbx for refraction index calculations "
+              "(no strict check)                                         %10s" %
+              ("pynx.gid.FhklDWBA5", s))
     if return_array:
         return fhkl
     return abs(fhkl_).mean() > 1
@@ -379,8 +386,8 @@ def speed(gpu_name, do_plot=False, language="OpenCL", cl_platform="", plot_style
              verticalalignment='center')
         text(1e5, 2e9, "$GPU$", fontsize=18, weight='bold', horizontalalignment='center', verticalalignment='center')
 
-        ylabel("$reflections\cdot atoms\cdot s^{-1}$", fontsize=18)
-        xlabel("$nb\ atoms$", fontsize=18)
+        ylabel(r"$reflections\cdot atoms\cdot s^{-1}$", fontsize=18)
+        xlabel(r"$nb\ atoms$", fontsize=18)
         xlim(30, 1e7)
         show()
 
@@ -454,7 +461,7 @@ class TestScattering(unittest.TestCase):
             import_ok = False
         self.assertTrue(import_ok, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.has_cython == False, "Cython is not available")
+    @unittest.skipIf(scattering.fhkl.has_cython is False, "Cython is not available")
     def test_fhkl_cpu(self):
         n = 20
 
@@ -482,29 +489,31 @@ class TestScattering(unittest.TestCase):
     @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhklo_cuda(self):
         n = 20
-        for p in scattering.fhkl.cl.get_platforms():
-            for d in p.get_devices():
-                msg = "Testing fhklo calculations: platform=OpenCL, device=%s" % d
-                res = fhklo(d.name, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
-                self.assertLess(res.mean(), 1, msg=msg)
+        scattering.fhkl.drv.init()
+        for i in range(scattering.fhkl.drv.Device.count()):
+            d = scattering.fhkl.drv.Device(i).name()
+            msg = "Testing fhklo calculations: platform=CUDA, device=%s" % d
+            res = fhklo(d, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+            self.assertLess(res.mean(), 1, msg=msg)
 
     @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhkl_graz_cuda(self):
         n = 20
-        for p in scattering.fhkl.cl.get_platforms():
-            for d in p.get_devices():
-                msg = "Testing fhkl_graz calculations: platform=OpenCL, device=%s" % d
-                res = fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
-                self.assertLess(res.mean(), 1, msg=msg)
+        scattering.fhkl.drv.init()
+        for i in range(scattering.fhkl.drv.Device.count()):
+            d = scattering.fhkl.drv.Device(i).name()
+            msg = "Testing fhkl_graz calculations: platform=OpenCL, device=%s" % d
+            res = fhkl_graz(d, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+            self.assertLess(res.mean(), 1, msg=msg)
 
     @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhklo_graz_cuda(self):
         n = 20
-        for p in scattering.fhkl.cl.get_platforms():
-            for d in p.get_devices():
-                msg = "Testing fhklo_graz calculations: platform=OpenCL, device=%s" % d
-                res = fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
-                self.assertLess(res.mean(), 1, msg=msg)
+        for i in range(scattering.fhkl.drv.Device.count()):
+            d = scattering.fhkl.drv.Device(i).name()
+            msg = "Testing fhklo_graz calculations: platform=OpenCL, device=%s" % d
+            res = fhklo_graz(d, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+            self.assertLess(res.mean(), 1, msg=msg)
 
     @unittest.skipIf(scattering.fhkl.cl is None, "PyOpenCL is not available")
     def test_fhkl_opencl(self):
