@@ -19,6 +19,7 @@ from pynx.cdi.test import suite as cdi_suite
 from pynx.cdi.runner.test import suite as cdi_runner_suite
 from pynx.ptycho.test import suite as ptycho_suite
 from pynx.ptycho.runner.test import suite as ptycho_runner_suite
+from pynx.scattering.test import suite as scattering_suite
 
 
 def suite():
@@ -29,6 +30,7 @@ def suite():
     test_suite.addTest(cdi_runner_suite())
     test_suite.addTest(ptycho_suite())
     test_suite.addTest(ptycho_runner_suite())
+    test_suite.addTest(scattering_suite())
     return test_suite
 
 
