@@ -33,7 +33,21 @@ class PynxOrderedDict(OrderedDict):
         Return dictionary as a numpy record array. Strings are converted to ASCII for h5py compatibility
         :return: the numpy record array, with two entries (cycle, value) per position
         """
-        a = np.rec.array([(k, v) for k, v in self.items()], names=('cycle', title))
+        kv = []
+        for k, v in self.items():
+            # Special handling for values like (2,2,2) condensed as 222
+            real_array = isinstance(v, np.ndarray)
+            if real_array:
+                if v.ndim < 1:
+                    real_array = False
+            if isinstance(v, tuple) or isinstance(v, list) or real_array:
+                if len(v) == 2:
+                    v = np.int16("%d%d" % (v[0], v[1]))
+                elif len(v) == 3:
+                    v = np.int16("%d%d%d" % (v[0], v[1], v[2]))
+            kv.append((k, v))
+
+        a = np.rec.array(kv, names=('cycle', title))
         tt = []
         for k, s in a.dtype.descr:
             if 'U' in s:
@@ -50,18 +64,16 @@ class History(dict):
     The default fields initialized as python OrderedDict (the key being the cycle number) are:
     
       - 'time': records timeit.default_timer()
-      - 'epoch': records time.time())
-
-    A special field is last_cycle
+      - 'epoch': records time.time()
     """
 
     def __init__(self):
         super(dict, self).__init__()
         for k in ['time', 'epoch']:
             self[k] = PynxOrderedDict()
-        self['time'][0] = timeit.default_timer()
-        self['epoch'][0] = time.time()
-        self.last_cycle = 0
+        # This records the beginning of the first algorithm
+        # To be useful, insert() should be called at the beginning of the algorithms
+        self.t0 = timeit.default_timer()
 
     def insert(self, cycle, **kwargs):
         """
@@ -74,11 +86,8 @@ class History(dict):
         Returns:
             Nothing.
         """
-        self.last_cycle = cycle
-        if cycle not in self['time']:
-            self['time'][cycle] = timeit.default_timer()
-        if cycle not in self['epoch']:
-            self['epoch'][cycle] = time.time()
+        self['time'][cycle] = timeit.default_timer()
+        self['epoch'][cycle] = time.time()
         for k, v in kwargs.items():
             if k not in self:
                 self[k] = PynxOrderedDict()
@@ -91,21 +100,35 @@ class History(dict):
         """
         Get stored values for one or several keys (e.g. 'time', 'llk', 'nb_photons') as a numpy record array.
         The first entry is always the cycle number. If entries are missing for a given key and a cycle number, it
-        is replaced by the last recorded value.
-        :param args: all the desired keys, e.g.: 'time', 'epoch', 'llk', 'algorithm'... If no args are given, all
-        available keys are returned.
-        :return: numpy record array, see https://docs.scipy.org/doc/numpy/user/basics.rec.html#record-arrays.
+        is replaced by the next recorded value.
 
+        :param args: all the desired keys, e.g.: 'time', 'epoch', 'llk', 'algorithm'... If no args are given, all
+          available keys are returned.
+        :return: numpy record array, see https://docs.scipy.org/doc/numpy/user/basics.rec.html#record-arrays,
+          or None if no history has been recorded
         """
         v = [[x] for x in self['time'].keys()]
+        if len(v) == 0:
+            return None
         if len(args) == 0:
             args = self.keys()
         # This is slow, but may be enough
         for k in args:
-            last_value = 0
-            for vx in v:
+            last_value = list(self[k].values())[-1]
+            # We use reversed because the first cycle is usually anomalous
+            for vx in reversed(v):
                 if vx[0] in self[k]:
                     last_value = self[k][vx[0]]
+                real_array = isinstance(last_value, np.ndarray)
+                if real_array:
+                    if last_value.ndim < 1:
+                        real_array = False
+                if isinstance(last_value, tuple) or isinstance(last_value, list) or real_array:
+                    if len(last_value) == 2:
+                        last_value = np.int16("%d%d" % (last_value[0], last_value[1]))
+                    elif len(last_value) == 3:
+                        last_value = np.int16("%d%d%d" % (last_value[0], last_value[1], last_value[2]))
+
                 vx.append(last_value)
         a = np.rec.array(v, names=['cycle'] + [k for k in args])
         # Convert unicode to ASCII arrays for h5py compatibility
@@ -126,8 +149,3 @@ if __name__ == '__main__':
     h.insert(15, algorithm='DM')
     a = h.as_numpy_record_array('llk', 'nb_photons', 'time', 'algorithm')
     b = h['algorithm'].as_numpy_record_array()
-    import h5py as h5
-
-    hh = h5.File('test.h5', 'w')
-    hh.create_dataset("a", data=a)
-    hh.create_dataset("b", data=b)
