@@ -287,9 +287,23 @@ def showCplx(im, mask=0, pixSize_um=1, showGrid=True, amplitudeLog=False, maskPh
 
 
 def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=100, pixel_size_object=None,
-                   scan_area_obj=None, scan_area_probe=None, scan_pos=None, **kwargs):
+                   scan_area_obj=None, scan_area_probe=None, scan_pos=None, invert_yaxis=False, **kwargs):
     """
-    Plots the progress of the ptychographic evaluation.
+    Live plot of the object and probe phase and amplitude. Only the first modes are shown.
+
+    :param obj: the object to plot
+    :param probe: the probe to plot
+    :param tit1: title for the object
+    :param tit2: title for the probe
+    :param stit: suptitle for the figure (e.g. the current algorithm, llk,...)
+    :param fig_num: the figure number to use, so plotting always appears in the same one
+    :param pixel_size_object: pixel size for the object
+    :param scan_area_obj: the 2D array of the object scan area (1 inside, 0 outside),
+        which will be sued to mask the object
+    :param scan_pos: a tuple (x,y) of the scan positions in meters.
+    :param invert_yaxis: if True, the plot will be inverted along the Y axis- this
+        is used for near field ptychography so the view corresponds to the full field
+        view, rather than using the motor axes as reference.
     """
     if 'minimize_obj_phase' in kwargs:
         warnings.warn("show_obj_probe: minimize_obj_phase is obsolete. Use ZeroPhaseRamp() operators",
@@ -343,6 +357,12 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
     else:
         scan_area_f = 1
 
+    if scan_pos is not None:
+        scan_posx = np.array(scan_pos[0] * scan_area_f)
+        scan_posx = np.append(scan_posx, scan_posx[0])
+        scan_posy = np.array(scan_pos[1] * scan_area_f)
+        scan_posy = np.append(scan_posy, scan_posy[0])
+
     ax0 = plt.subplot(gs[0])
     ny, nx = obj.shape
     if pixel_size_object is None:
@@ -357,7 +377,7 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
         plt.ylabel(u"y(µm)")
         ax0.xaxis.set_label_coords(1.05, -.05)
     if scan_pos is not None:
-        ax0.plot(scan_pos[0] * scan_area_f, scan_pos[1] * scan_area_f, 'k-', linewidth=0.5)
+        ax0.plot(scan_posx, scan_posy, 'k-', linewidth=0.5)
     plt.title(tit1 + ' modulus')
 
     ax1 = plt.subplot(gs[1])
@@ -373,8 +393,11 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
         plt.ylabel(u"y(µm)")
         ax1.xaxis.set_label_coords(1.05, -.05)
     if scan_pos is not None:
-        ax1.plot(scan_pos[0] * scan_area_f, scan_pos[1] * scan_area_f, 'k-', linewidth=0.5)
-    plt.title(tit1 + ' phase')
+        ax1.plot(scan_posx, scan_posy, 'k-', linewidth=0.5)
+    if obj_phi_min is None:
+        plt.title(tit1 + ' phase')
+    else:
+        plt.title(tit1 + ' phase [%5.2f-%5.2f radians]' % (obj_phi_min, obj_phi_max))
 
     ax2 = plt.subplot(gs[2])
     ny, nx = probe.shape
@@ -391,7 +414,7 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
         ax2.xaxis.set_label_coords(1.05, -.05)
         plt.xticks(rotation=30)
     if scan_pos is not None:
-        ax2.plot(scan_pos[0] * scan_area_f, scan_pos[1] * scan_area_f, 'k-', linewidth=0.5)
+        ax2.plot(scan_posx, scan_posy, 'k-', linewidth=0.5)
     plt.title(tit2 + ' modulus')
 
     ax3 = plt.subplot(gs[3])
@@ -409,8 +432,15 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
         ax3.xaxis.set_label_coords(1.05, -.05)
         plt.xticks(rotation=30)
     if scan_pos is not None:
-        ax3.plot(scan_pos[0] * scan_area_f, scan_pos[1] * scan_area_f, 'k-', linewidth=0.5)
-    plt.title(tit2 + ' phase')
+        ax3.plot(scan_posx, scan_posy, 'k-', linewidth=0.5)
+    if probe_phi_min is None:
+        plt.title(tit2 + ' phase')
+    else:
+        plt.title(tit2 + ' phase [%5.2f-%5.2f radians]' % (probe_phi_min, probe_phi_max))
+
+    if invert_yaxis:
+        for ax in [ax0, ax1, ax2, ax3]:
+            ax.invert_yaxis()
 
     if stit:
         txt = fig.suptitle('')
@@ -419,6 +449,7 @@ def show_obj_probe(obj, probe, tit1='Object', tit2='Probe', stit=None, fig_num=1
     try:
         plt.draw()
         plt.gcf().canvas.draw()
-        plt.pause(.001)
+        if 'ipympl' not in plt.get_backend():  # This outputs the graphs again
+            plt.pause(.001)
     except:
         pass
