@@ -6,11 +6,15 @@
 #         Vincent Favre-Nicolin, favre@esrf.fr
 
 import timeit
+from packaging.version import parse as version_parse
 import numpy as np
 from scipy.ndimage import fourier_shift
 
-from scipy import log10, sqrt, pi, exp
-from scipy.fftpack import ifftshift, fftshift, fft2, ifft2, fftfreq, fftn, ifftn
+from scipy.fft import ifftshift, fftshift, rfftn, rfftfreq, fftfreq, fftn, ifftn
+from scipy.special import erfc
+# from skimage.registration import phase_cross_correlation as phase_cross_correlation_orig
+# from skimage import __version__ as skimage_version
+from ._phase_cross_correlation import phase_cross_correlation as phase_cross_correlation_orig
 
 from pynx import scattering
 from ..processing_unit import has_cuda, has_opencl
@@ -23,7 +27,6 @@ if has_cuda:
     from pycuda.elementwise import ElementwiseKernel as CU_Elk
     import pycuda.tools as cu_tools
     from pycuda.compiler import SourceModule
-    import skcuda.fft as cu_fft
 
     argmax_dtype = np.dtype([("idx", np.int32), ("cur_max", np.float32)])
     cu_tools.get_or_register_dtype("idx_max", argmax_dtype)
@@ -62,6 +65,69 @@ def shift(img, v):
         return ifftn(fourier_shift(fftn(img.astype(dtype)), v))
 
 
+def phase_cross_correlation(reference_image, moving_image, low_cutoff=None, low_width=0.03,
+                            high_cutoff=None, high_width=0.03, **kwargs):
+    """
+    phase cross-correlation from scikit-image. This version automatically adds
+    the normalization keyword for skimage>=0.19, and adds low and/or high-bandpass
+    filters.
+
+    :param reference_image: the reference image
+    :param moving_image: the moving image
+    :param low_cutoff: a 0<value<<0.5 can be given (typically it should be a few 0.01),
+        an erfc filter with a cutoff at low_cutoff*N (where N is the size along each dimension)
+        will be applied, after the images have been FT'd
+    :param low_width: the width of the low cutoff filter, also as a percentage of the size
+    :param high_cutoff: same as low_cutoff fot the high frequency filter, should be close below 0.5
+    :param high_width: same as low_width
+    :param kwargs: keyword arguments which will be passed to skimage's phase_cross_correlation
+    :return:
+    """
+    # if version_parse(skimage_version) >= version_parse('0.19'):
+    if 'normalization' not in kwargs:
+        kwargs['normalization'] = None
+
+    sh = reference_image.shape
+
+    assert np.allclose(sh, moving_image.shape)
+
+    if low_cutoff is not None or high_cutoff is not None:
+        need_ft = True
+        r2c = kwargs['r2c'] if 'r2c' in kwargs else False
+        if 'space' in kwargs:
+            if 'real' == kwargs['space']:
+                need_ft = False
+        if need_ft:
+            if np.isrealobj(reference_image) and np.isrealobj(moving_image):
+                reference_image = rfftn(reference_image)
+                moving_image = rfftn(moving_image)
+                r2c = True
+                kwargs['r2c'] = True
+            else:
+                reference_image = fftn(reference_image)
+                moving_image = fftn(moving_image)
+            kwargs['space'] = 'fourier'
+        r = 0
+        for i in range(len(sh)):
+            if r2c and i == len(sh)-1:
+                tmp = rfftfreq(sh[i]).astype(np.float32)
+            else:
+                tmp = fftfreq(sh[i]).astype(np.float32)
+            for ii in range(len(sh) - i - 1):
+                tmp = np.expand_dims(tmp, axis=1)
+            r = r + tmp ** 2
+        r = np.sqrt(r)
+        if low_cutoff:
+            tmp = 1 - 0.5 * erfc((r - low_cutoff) / low_width)
+            reference_image *= tmp
+            moving_image *= tmp
+        if high_cutoff:
+            tmp = 0.5 * erfc((r - high_cutoff) / high_width)
+            reference_image *= tmp
+            moving_image *= tmp
+    return phase_cross_correlation_orig(reference_image, moving_image, **kwargs)
+
+
 def register_translation_old(ref_img, img, upsampling=1, verbose=False, gpu='CPU'):
     """
     Calculate the translation shift between two images.
@@ -206,20 +272,19 @@ def register_translation_cuda(ref_img, img, upsampling=10, processing_unit=None,
                     options=["-use_fast_math"],
                     arguments="float *d")
 
-    fft_plan = pu.cu_fft_get_plan(ref_img, batch=True)
     if overwrite is False:
         d1f, d2f = cua.empty_like(ref_img), cua.empty_like(ref_img)
         # out-of-place FFT of the two images
-        cu_fft.fft(ref_img, d1f, fft_plan, scale=True)
-        cu_fft.fft(img, d2f, fft_plan, scale=True)
+        pu.fft(ref_img, d1f, norm=True)
+        pu.fft(img, d2f, norm=True)
     else:
         d1f, d2f = ref_img, img
-        cu_fft.fft(d1f, d1f, fft_plan, scale=True)
-        cu_fft.fft(d2f, d2f, fft_plan, scale=True)
+        pu.fft(d1f, d1f, norm=True)
+        pu.fft(d2f, d2f, norm=True)
 
     # Pixel registration
     cc0 = d1f * d2f.conj()
-    cu_fft.ifft(cc0, d1f, fft_plan, scale=False)
+    pu.ifft(cc0, d1f, norm=False)
     idx = register_translation_cuda.cu_argmax_c_red(d1f).get()["idx"]
 
     ny, nx = d1f.shape
@@ -316,18 +381,15 @@ def register_translation_cuda_n(ref_img, img, upsampling=10, processing_unit=Non
                     options=["-use_fast_math"],
                     arguments="pycuda::complex<float> *d")
 
-    fft_plan1 = pu.cu_fft_get_plan(ref_img, batch=True)
-    fft_plan2 = pu.cu_fft_get_plan(img, batch=True)
-    fft_plan0 = pu.cu_fft_get_plan(img[0], batch=True)
     if overwrite is False:
         d1f, d2f = cua.empty_like(ref_img), cua.empty_like(img)
         # out-of-place FFT of the two images
-        cu_fft.fft(ref_img, d1f, fft_plan1, scale=True)
-        cu_fft.fft(img, d2f, fft_plan2, scale=True)
+        pu.fft(ref_img, d1f, ndim=2, norm=True)
+        pu.fft(img, d2f, ndim=2, norm=True)
     else:
         d1f, d2f = ref_img, img
-        cu_fft.fft(d1f, d1f, fft_plan1, scale=True)
-        cu_fft.fft(d2f, d2f, fft_plan2, scale=True)
+        pu.fft(d1f, d1f, ndim=2, norm=True)
+        pu.fft(d2f, d2f, ndim=2, norm=True)
 
     # Pixel registration
     nzref, ny, nx = ref_img.shape
@@ -338,7 +400,7 @@ def register_translation_cuda_n(ref_img, img, upsampling=10, processing_unit=Non
     for i in range(nz):
         i1 = i * nzref // nz
         d2f[i] = d1f[i1] * d2f[i].conj()
-        cu_fft.ifft(d2f[i], cutmp, fft_plan0, scale=False)
+        pu.ifft(d2f[i], cutmp, ndim=2, norm=False)
         idx = register_translation_cuda.cu_argmax_c_red(cutmp).get()["idx"]
         dy0, dx0 = idx // nx, idx % nx
         dy[i] = dy0 - (dy0 > ny / 2) * ny
